@@ -3,9 +3,9 @@
 
 ;; Author: NAKAJIMA Mikio <minakaji@namazu.org>
 ;; Maintainer: NAKAJIMA Mikio <minakaji@namazu.org>
-;; Version: $Id: skk-rdbms.el,v 1.11 2007/07/21 08:29:30 skk-cvs Exp $
+;; Version: $Id: skk-rdbms.el,v 1.12 2007/09/06 13:50:22 skk-cvs Exp $
 ;; Keywords: japanese, rdbms
-;; Last Modified: $Date: 2007/07/21 08:29:30 $
+;; Last Modified: $Date: 2007/09/06 13:50:22 $
 
 ;; This file is not part of Daredevil SKK yet.
 
@@ -207,92 +207,85 @@ CREATE TABLE hinsi_data \(
 ;; $B$+$i$M!#(BPostgreSQL $B0J30$N(B RDBMS $B$r;H$&?M$K$O%^%/%m$r=q$-BX$($F$b$i(B
 ;; $B$&(B...$B9s$@$m$&$+!)(B
 (defmacro skk-rdbms-SQL-insert-command (word)
-  (`
-   (if (not skk-henkan-okurigana)
+  `(if (not skk-henkan-okurigana)
        (format
 	"INSERT INTO %s (okuriari,yomi,kanji,date) VALUES (%s, '%s', '%s', '%s');"
-	skk-rdbms-private-jisyo-table 0 skk-henkan-key (, word)
+	skk-rdbms-private-jisyo-table 0 skk-henkan-key ,word
 	(current-time-string))
      (format
       "INSERT INTO %s (okuriari,yomi,kanji,okurigana,date) VALUES (%s, '%s', '%s', '%s', '%s');"
-      skk-rdbms-private-jisyo-table 1 skk-henkan-key (, word) skk-henkan-okurigana
-      (current-time-string)))))
+      skk-rdbms-private-jisyo-table 1 skk-henkan-key ,word skk-henkan-okurigana
+      (current-time-string))))
 
 (defmacro skk-rdbms-SQL-delete-command (word)
-  (`
-   (concat
+  `(concat
     (format
      "DELETE FROM %s WHERE kanji = '%s' AND yomi = '%s' AND okuriari = %s "
-	    skk-rdbms-private-jisyo-table (, word) skk-henkan-key
-	    (if skk-okuri-char 1 0))
+     skk-rdbms-private-jisyo-table ,word skk-henkan-key
+     (if skk-okuri-char 1 0))
     (if (and skk-henkan-okuri-strictly skk-henkan-okurigana)
 	(format "AND okurigana = '%s'" skk-henkan-okurigana))
-    ";")))
+    ";"))
 
 (defmacro skk-rdbms-SQL-regexp-delete-command (word)
-  (`
-   (concat
+  `(concat
     ;; $BEvA3$@$1$I!"@55,I=8=$,;H$($J$$(B RDBMS $B$G$O;H$($J$$$M(B...$B!#$I$s(B
     ;; $B$I$s(B PostgreSQL $B4s$j$K$J$C$F$f$/(B...f(^_^;;;$B!#(B
     (format
      "DELETE FROM %s WHERE kanji ~ '%s' AND yomi = '%s' AND okuriari = %s "
-	    skk-rdbms-private-jisyo-table (, word) skk-henkan-key
-	    (if skk-okuri-char 1 0))
+     skk-rdbms-private-jisyo-table ,word skk-henkan-key
+     (if skk-okuri-char 1 0))
     (if (and skk-henkan-okuri-strictly skk-henkan-okurigana)
 	(format "AND okurigana = '%s'" skk-henkan-okurigana))
-    ";")))
+    ";"))
 
 (defmacro skk-rdbms-SQL-update-command (word)
-  (`
-   (concat
+  `(concat
     ;; SET date = 'now'::abstime $B$C$F(B PostgreSQL $B$@$1$G$7$+DLMQ$7$J$$%3(B
     ;; $B%^%s%I!)(B  SQL92 ($B$H$$$&$N$+$J!)(B) $B$G$O$I$&=q$/$s$G$7$g$&!)(B
     (format
      "UPDATE %s SET date = 'now'::abstime WHERE kanji = '%s' AND yomi = '%s' AND okuriari = %s "
-	    skk-rdbms-private-jisyo-table (, word) skk-henkan-key (if skk-okuri-char 1 0))
+     skk-rdbms-private-jisyo-table ,word skk-henkan-key (if skk-okuri-char 1 0))
     (if (and skk-henkan-okuri-strictly skk-henkan-okurigana)
 	(format "AND okurigana = '%s'" skk-henkan-okurigana))
-    ";")))
+    ";"))
 
 (defmacro skk-rdbms-SQL-search-jisyo-command (table)
-  (`
-   (format "SELECT kanji FROM %s WHERE yomi = '%s' AND okuriari = %s;"
-	   (, table) skk-henkan-key (if skk-okuri-char 1 0))))
+  `(format "SELECT kanji FROM %s WHERE yomi = '%s' AND okuriari = %s;"
+	   ,table skk-henkan-key (if skk-okuri-char 1 0)))
 
 (defmacro skk-rdbms-SQL-search-private-jisyo-command ()
-  (`
-   (concat
+  `(concat
     (format "SELECT kanji FROM %s WHERE yomi = '%s' AND okuriari = %s "
 	    skk-rdbms-private-jisyo-table skk-henkan-key (if skk-okuri-char 1 0))
     (if (and skk-henkan-okurigana skk-henkan-okuri-strictly)
 	(format "AND okurigana = '%s' " skk-henkan-okurigana))
-    "ORDER BY date DESC;")))
+    "ORDER BY date DESC;"))
 
 (defmacro skk-rdbms-SQL-search-completion-word-command ()
-  (`
-   (format
+  `(format
     "SELECT yomi FROM %s WHERE yomi LIKE '%s%s' AND okuriari = 0 GROUP BY yomi ORDER BY max(date) DESC;"
-    skk-rdbms-private-jisyo-table skk-completion-word skk-rdbms-SQL-wildcard)))
+    skk-rdbms-private-jisyo-table skk-completion-word skk-rdbms-SQL-wildcard))
 
 (defmacro skk-rdbms-SQL-search-busyu-command (key)
-  (` (format
-      ;; SQL $BJ8$NESCf$K2~9T$rF~$l$k$H(B psql $B$NBh(B 2 $B%W%m%s%W%H$,=P$k!#$3$l$r@55,I=8=$G(B
-      ;; $BI=8=$7$F8!:w$9$k$H%*!<%P!<%X%C%I$,Bg$-$/!"3NDj$7$F$+$i$N<-=q99?7$,CY$/$J$k!#(B
-      ;; $B=>$$(B SQL $BJ8$OD9$/$F$bL5M}LpM}(B 1 $B9T$GI=8=$9$k$+!"7y$J$i:Y@Z$l$K@Z$C$F(B
-      ;; concat $B$G$D$J$0!#(B
-      "SELECT kanji FROM %s WHERE busyuID = (SELECT busyuID FROM %s WHERE yomi1 = '%s' OR yomi2 = '%s' OR yomi3 = '%s' OR yomi4 = '%s');"
-      skk-rdbms-busyu-data-table skk-rdbms-busyu-base-table
-      (, key) (, key) (, key) (, key))))
+  `(format
+    ;; SQL $BJ8$NESCf$K2~9T$rF~$l$k$H(B psql $B$NBh(B 2 $B%W%m%s%W%H$,=P$k!#$3$l$r@55,I=8=$G(B
+    ;; $BI=8=$7$F8!:w$9$k$H%*!<%P!<%X%C%I$,Bg$-$/!"3NDj$7$F$+$i$N<-=q99?7$,CY$/$J$k!#(B
+    ;; $B=>$$(B SQL $BJ8$OD9$/$F$bL5M}LpM}(B 1 $B9T$GI=8=$9$k$+!"7y$J$i:Y@Z$l$K@Z$C$F(B
+    ;; concat $B$G$D$J$0!#(B
+    "SELECT kanji FROM %s WHERE busyuID = (SELECT busyuID FROM %s WHERE yomi1 = '%s' OR yomi2 = '%s' OR yomi3 = '%s' OR yomi4 = '%s');"
+    skk-rdbms-busyu-data-table skk-rdbms-busyu-base-table
+    ,key ,key ,key ,key))
 
 (defmacro skk-rdbms-SQL-search-stroke-command (stroke)
-  (` (format "SELECT kanji FROM %s WHERE stroke = %s;"
-	     skk-rdbms-stroke-table (, stroke))))
+  `(format "SELECT kanji FROM %s WHERE stroke = %s;"
+	   skk-rdbms-stroke-table ,stroke))
 
 (defmacro skk-rdbms-search-sahen-command (dakuten)
-  (` (format
-      "SELECT kanji FROM %s WHERE yomi = '%s' AND hinsiID = (SELECT hinsiID FROM %s WHERE hinsi = '%s');"
-      skk-rdbms-hinsi-data-table skk-henkan-key skk-rdbms-hinsi-base-table
-      (if dakuten "$B%69T(B($B$:$k(B)" "$B%59T(B($B$9$k(B)"))))
+  `(format
+    "SELECT kanji FROM %s WHERE yomi = '%s' AND hinsiID = (SELECT hinsiID FROM %s WHERE hinsi = '%s');"
+    skk-rdbms-hinsi-data-table skk-henkan-key skk-rdbms-hinsi-base-table
+    (if dakuten "$B%69T(B($B$:$k(B)" "$B%59T(B($B$9$k(B)")))
 
 (defvar skk-rdbms-cutoff-output-function
   (function
