@@ -61,11 +61,16 @@
     ("$B%[(B" "$B%\(B" "$B%](B")) "\
 $BByE@$HH>ByE@$rF~NO$9$k$?$a$N%k!<%k!#(B")
 
+(defconst skk-kanagaki-print-help-function
+  (cond ((memq skk-emacs-type '(mule6))
+	 #'help-print-return-message)
+	(t
+	 #'print-help-return-message)))
+
 ;;;###autoload
 (defmacro skk-kanagaki-help-1 (bufname title list)
   `(let ((buf (get-buffer-create ,bufname)))
-     (save-excursion
-       (set-buffer buf)
+     (with-current-buffer buf
        (setq buffer-read-only nil)
        (erase-buffer)
        (insert
@@ -89,7 +94,7 @@
        (goto-char (point-min))
        (help-mode))
      (let ((standard-output buf))
-       (print-help-return-message))
+       (funcall skk-kanagaki-print-help-function))
      (display-buffer buf)))
 
 ;;;###autoload
