@@ -50,7 +50,7 @@
 (eval-and-compile
   (require 'skk-kanagaki))
 
-(static-when (eq skk-emacs-type 'mule5)
+(static-when (memq skk-emacs-type '(mule5 mule6))
   (eval-and-compile
     (require 'skk-e21)))
 
@@ -224,27 +224,7 @@
 ;; Hooks.
 
 (add-hook 'skk-mode-hook 'skk-nicola-setup)
-
-(add-hook
- 'skk-mode-hook
- #'(lambda ()
-     ;;
-     (case skk-kanagaki-state
-       (kana
-	(setq skk-hiragana-mode-string skk-nicola-hiragana-mode-string
-	      skk-katakana-mode-string skk-nicola-katakana-mode-string))
-       (rom
-	(setq skk-hiragana-mode-string skk-nicola-hiragana-rom-string
-	      skk-katakana-mode-string skk-nicola-katakana-rom-string)))
-     ;;
-     (skk-modify-indicator-alist 'katakana
-				 skk-katakana-mode-string)
-     (skk-modify-indicator-alist 'hiragana
-				 skk-hiragana-mode-string)
-     ;;
-     (skk-update-modeline (if skk-katakana
-			      'katakana
-			    'hiragana))))
+(add-hook 'skk-mode-hook 'skk-nicola-setup-modeline)
 
 ;; Functions.
 
@@ -296,6 +276,24 @@
   ;;
   (remove-hook 'skk-mode-hook 'skk-nicola-setup))
 
+(defun skk-nicola-setup-modeline ()
+  (case skk-kanagaki-state
+    (kana
+     (setq skk-hiragana-mode-string skk-nicola-hiragana-mode-string
+	   skk-katakana-mode-string skk-nicola-katakana-mode-string))
+    (rom
+     (setq skk-hiragana-mode-string skk-nicola-hiragana-rom-string
+	   skk-katakana-mode-string skk-nicola-katakana-rom-string)))
+  ;;
+  (skk-modify-indicator-alist 'katakana
+			      skk-katakana-mode-string)
+  (skk-modify-indicator-alist 'hiragana
+			      skk-hiragana-mode-string)
+  ;;
+  (skk-update-modeline (if skk-katakana
+			   'katakana
+			 'hiragana)))
+
 (defun skk-nicola-setup-tutorial ()
   (dolist (key skk-nicola-lshift-keys)
     (define-key skktut-j-mode-map
@@ -425,6 +423,13 @@
   ;; verbose message
   (skk-henkan-on-message))
 
+;;;###autoload
+(let ((property (if (featurep 'xemacs)
+		    'pending-del
+		  'delete-selection)))
+  (put 'skk-nicola-self-insert-rshift property t)
+  (put 'skk-nicola-self-insert-lshift property t))
+
 (defun skk-nicola-self-insert-lshift-1 (arg parg)
   (when (or (and (markerp skk-nicola-okuri-flag)
 		 (<= (point)
@@ -437,8 +442,8 @@
 	 (not skk-jisx0201-mode))
     (skk-nicola-insert arg parg))
    (t
-    (unless last-command-char
-      (setq last-command-char ?\ ))
+    (unless (skk-last-command-char)
+      (skk-set-last-command-char ?\ ))
     (call-interactively 'skk-insert))))
 
 ;;;###autoload
@@ -448,11 +453,11 @@
   (interactive "*p")
   (if (skk-sit-for skk-nicola-latin-interval t)
       ;; then
-      (let ((last-command-char
-	     (if (characterp (event-to-character
-			      last-command-event))
-		 (event-to-character last-command-event)
-	       ?\ )))
+      (skk-bind-last-command-char
+	  (if (characterp (event-to-character
+			   last-command-event))
+	      (event-to-character last-command-event)
+	    ?\ )
 	(call-interactively 'self-insert-command t))
     ;; else
     (let ((last (static-cond
@@ -468,18 +473,18 @@
 	  char)
       (if (eq last next)
 	  ;; then
-	  (let ((last-command-char
-		 (if (characterp (event-to-character
-				  last-command-event))
-		     (event-to-character last-command-event)
-		   ?\ )))
+	  (skk-bind-last-command-char
+	      (if (characterp (event-to-character
+			       last-command-event))
+		  (event-to-character last-command-event)
+		?\ )
 	    (call-interactively 'self-insert-command t)
 	    (call-interactively 'self-insert-command t))
 	;; else
 	(when (characterp next)
 	  (setq char next)
 	  (setq next (key-description
-		      (char-to-string char))))
+		      (skk-char-to-unibyte-string char))))
 	(when (eq next 'space)
 	  (setq next (key-description " ")))
 	(when (symbolp next)
@@ -500,14 +505,13 @@
 	       (skk-j-mode-on)
 	       (skk-cursor-set skk-cursor-hiragana-color))
 	      (char
-	       (let ((last-command-char
-		      (if (characterp (event-to-character
-				       last-command-event))
-			  (event-to-character
-			   last-command-event)
-			?\ )))
+	       (skk-bind-last-command-char
+		   (if (characterp (event-to-character
+				    last-command-event))
+		       (event-to-character last-command-event)
+		     ?\ )
 		 (call-interactively 'self-insert-command t))
-	       (let ((last-command-char char))
+	       (skk-bind-last-command-char char
 		 (call-interactively 'self-insert-command
 				     t)))))))
   nil)
@@ -549,7 +553,7 @@
     ;; $BE}7WE*2ACM$,$"$k$+$J(B...$B!)(B
 ;    (setq skk-nicola-temp-data
 ;	  (cons
-;	   (list (or last-command-char this-command)
+;	   (list (or (skk-last-command-char) this-command)
 ;		 period1
 ;		 next
 ;		 period2
@@ -561,7 +565,7 @@
 
 (defun skk-nicola-format-time (time)
   "`current-time' $B$NJV$97k2L$rJQ49$7$FI>2A$G$-$k$h$&$K$9$k!#(B"
-  (let ((time1 (* (float 100000)
+  (let ((time1 (* (float 65536) ;; 2x10^6
 		  (car time)))
 	(time2 (cadr time))
 	(time3 (/ (caddr time)
@@ -625,7 +629,7 @@
 				  (current-time)))
 		     time2)
 	  str (if (characterp next)
-		  (char-to-string next))
+		  (skk-char-to-unibyte-string next))
 	  third-event (next-command-event)
 	  third (skk-nicola-event-to-key third-event))
     (cond
@@ -652,7 +656,7 @@
 
 (defun skk-nicola-insert-single (command arg &optional parg)
   "$BC1FHBG80$r=hM}$9$k!#(B"
-  (let ((char last-command-char))
+  (let ((char (skk-last-command-char)))
     (case command
       (skk-nicola-self-insert-rshift
        ;; ($BJQ49!&%9%Z!<%9(B)
@@ -673,14 +677,14 @@
 		   first)
 		  ((characterp first)
 		   (lookup-key skk-j-mode-map
-			       (char-to-string first)))
+			       (skk-char-to-unibyte-string first)))
 		  (t
 		   (lookup-key skk-j-mode-map first))))
 	(char (if (characterp first)
 		  first
-		last-command-char))
+		(skk-last-command-char)))
 	(str (when (characterp next)
-	       (char-to-string next))))
+	       (skk-char-to-unibyte-string next))))
     ;;
     (case (lookup-key skk-j-mode-map (or str next))
       (skk-nicola-self-insert-rshift
@@ -688,7 +692,7 @@
        (case command
 	 (skk-nicola-self-insert-rshift
 	  ;; [$B1&(B $B1&(B]
-	  (let ((last-command-char ?\ ))
+	  (skk-bind-last-command-char ?\ 
 	    (cond (skk-henkan-mode
 		   ;;
 		   (skk-kanagaki-insert arg)
@@ -744,7 +748,7 @@
 				 skk-nicola-lshift-rule
 				 arg))
 	((and (not (eq char next))
-	      (memq last-command-char
+	      (memq (skk-last-command-char)
 		    skk-nicola-set-henkan-point-chars)
 	      (memq next
 		    skk-nicola-set-henkan-point-chars))
@@ -841,14 +845,14 @@
 		   first)
 		  ((characterp first)
 		   (lookup-key skk-j-mode-map
-			       (char-to-string first)))
+			       (skk-char-to-unibyte-string first)))
 		  (t
 		   (lookup-key skk-j-mode-map first))))
 	(char (if (characterp first)
 		  first
-		last-command-char))
+		(skk-last-command-char)))
 	(str (when (characterp next)
-	       (char-to-string next)))
+	       (skk-char-to-unibyte-string next)))
 	(shifts '(skk-nicola-self-insert-lshift
 		  skk-nicola-self-insert-rshift)))
   (or
@@ -861,7 +865,7 @@
    (and (not (eq char next))
 	(or
 	 ;; [fj]
-	 (and (memq last-command-char
+	 (and (memq (skk-last-command-char)
 		    skk-nicola-set-henkan-point-chars)
 	      (memq next
 		    skk-nicola-set-henkan-point-chars))
@@ -953,7 +957,7 @@ ARG $B$rM?$($i$l$?>l9g$O$=$N?t$@$1J8;zNs$rO"7k$7$FF~NO$9$k!#(B"
 
 (defun skk-nicola-space-function (&optional arg parg)
   "$B?F;X1&%-!<C1FHBG80;~$N5sF0$r7h$a$k4X?t!#(B"
-  (let ((last-command-char ?\ ))
+  (skk-bind-last-command-char ?\ 
     (cond
      ((eq skk-henkan-mode 'active)
       (call-interactively 'skk-insert))
@@ -982,6 +986,11 @@ ARG $B$rM?$($i$l$?>l9g$O$=$N?t$@$1J8;zNs$rO"7k$7$FF~NO$9$k!#(B"
 
 ;; Pieces of Advice.
 
+(defadvice skk-kanagaki-initialize (after skk-nicols-setup activate)
+  ;; M-x skk-restart $BBP:v$H$7$F(B
+  (add-hook 'skk-mode-hook 'skk-nicola-setup)
+  (add-hook 'skk-mode-hook 'skk-nicola-setup-modeline))
+
 (defadvice skk-insert (before skk-nicola-update-flag activate)
   "$BAw$jBT$A>uBV$r4IM}$9$k!#(B"
   (when (or (and (markerp skk-nicola-okuri-flag)
@@ -1024,9 +1033,9 @@ ARG $B$rM?$($i$l$?>l9g$O$=$N?t$@$1J8;zNs$rO"7k$7$FF~NO$9$k!#(B"
     (cond
      ((and (eq skk-kanagaki-state 'kana)
 	   skk-j-mode
-	   (or (eq last-command-char
+	   (or (eq (skk-last-command-char)
 		   (car cell1))
-	       (eq last-command-char
+	       (eq (skk-last-command-char)
 		   (car cell2)))
 	   skk-henkan-mode)
       ;; $B$J$<$+$3$1$k!#860x2rL@Cf!#(B
@@ -1035,9 +1044,9 @@ ARG $B$rM?$($i$l$?>l9g$O$=$N?t$@$1J8;zNs$rO"7k$7$FF~NO$9$k!#(B"
 	(setq marker skk-henkan-start-point)
 	(skk-kakutei)
 	ad-do-it
-	(unless (or (string= (skk-char-to-string (char-before))
+	(unless (or (string= (char-to-string (char-before))
 			     (cadr cell1))
-		    (string= (skk-char-to-string (char-before))
+		    (string= (char-to-string (char-before))
 			     (cadr cell2)))
 	  (skk-save-point
 	   (goto-char marker)
