@@ -1,14 +1,14 @@
 ;;; skk-annotation.el --- SKK annotation $B4XO"%W%m%0%i%`(B -*- coding: iso-2022-jp -*-
 
 ;; Copyright (C) 2000, 2001 NAKAJIMA Mikio <minakaji@osaka.email.ne.jp>
-;; Copyright (C) 2000-2007  SKK Development Team <skk@ring.gr.jp>
+;; Copyright (C) 2000-2008  SKK Development Team <skk@ring.gr.jp>
 
 ;; Author: NAKAJIMA Mikio <minakaji@osaka.email.ne.jp>
 ;; Maintainer: SKK Development Team <skk@ring.gr.jp>
-;; Version: $Id: skk-annotation.el,v 1.133 2007/08/17 14:36:08 skk-cvs Exp $
+;; Version: $Id: skk-annotation.el,v 1.146 2010/01/27 03:02:48 skk-cvs Exp $
 ;; Keywords: japanese, mule, input method
 ;; Created: Oct. 27, 2000.
-;; Last Modified: $Date: 2007/08/17 14:36:08 $
+;; Last Modified: $Date: 2010/01/27 03:02:48 $
 
 ;; This file is part of Daredevil SKK.
 
@@ -110,20 +110,6 @@
 ;;   M-x skk-annotation-remove
 ;;
 ;; $B$7$F2<$5$$!#(B
-;; `;' $B$NJ8;z$r4^$s$@8uJd$O!"(Beval $B$9$k$H(B `;' $B$K$J$k(B Lisp $B<0$H$7$F(B
-;; quote $B$5$l$F<-=q8uJd$H$7$F<}$a$i$l$J$1$l$P$J$j$^$;$s!#4{B8$N<-=q$K(B
-;; $B$D$$$F$O!"<-=q$rFI$_9~$s$@%P%C%U%!$G(B
-;;
-;;   M-x skk-annotation-update-jisyo-format
-;;
-;; $B$9$k$3$H$G$3$N:n6H$r9T$J$&$3$H$,$G$-$^$9!#8D?M<-=q!"(BSKK-JISYO.L $B$K(B
-;; $B$D$$$F$O@'Hs9T$J$C$F$*$$$?J}$,NI$$$G$7$g$&!#(B
-;; SKK Openlab $B$G:#8eG[I[$9$k<-=q$O(B `;' $B$OM=$a(B quote $B$5$l$F$$$k>uBV$K(B
-;; $B$7$^$9!#(B
-;; $BC"$7!"4{$K%"%N%F!<%7%g%s$,IU$1$i$l$F$$$k>l9g$O!"$3$N%"%N%F!<%7%g%s(B
-;; $B<+BN$b8uJd$H6hJL$G$-$:$K(B quote $B$5$l$F$7$^$$$^$9$N$G!"$4Cm0U2<$5$$(B
-;; ($B:#$N$H$3$m<j:n6H$G(B quote $B$5$l$J$$$h$&$KB`Hr$9$k$J$I$7$+J}K!$O$"$j(B
-;; $B$^$;$s(B)$B!#(B
 ;;
 ;; Viper $BBP:v$O$^$@9T$J$C$F$$$^$;$s!#(B~/.viper $B$K<!$N$h$&$K=q$$$F2<$5$$!#(B
 ;; (viper-harness-minor-mode "skk-annotation")
@@ -173,6 +159,57 @@
 ;;    http://www.meadowy.org/~shirai/
 ;;
 ;;    $B$+$i:G?7HG$,F~<j$G$-$^$9!#(B
+;;
+;; <$B5l$$(B SKK $B$+$i$N0\9T(B>
+;;
+;; $B$3$N9`$O%"%N%F!<%7%g%s5!G=$,$J$$5l$$(B SKK (DDSKK 11.2 $B0JA0$^$?$O(B SKK
+;; 10.62 $B0JA0(B) $B$+$i:G?7$N$b$N$K0\9T$9$k>l9g$NCm0U;v9`$G$9!#(B
+;;
+;; $B%"%N%F!<%7%g%s$O%;%Q%l!<%?$H$7$F(B `;' $B$r;HMQ$7$F$$$k$?$a!"(B`;' $B$NJ8;z(B
+;; $B$r4^$s$@8uJd$O!"(Beval $B$9$k$H(B `;' $B$K$J$k(B Lisp $B<0$H$7$F(B quote $B$7<-=q8u(B
+;; $BJd$K<}$a$kI,MW$,$"$j$^$9!#(B
+;;
+;; $B$^$@%"%N%F!<%7%g%s5!G=$r0lEY$b;HMQ$7$F$$$J$$8D?M<-=q$K$D$$$F$O!"0J2<(B
+;; $B$N(B S $B<0$rI>2A$7$?8e!"(B
+;;
+;;   (defun skk-annotation-update-jisyo-format ()
+;;     (interactive)
+;;     (skk-setup-jisyo-buffer)
+;;     (let ((min skk-okuri-ari-min) (max skk-okuri-ari-max))
+;;       (skk-annotation-update-jisyo-format-1 min max)
+;;       (setq min skk-okuri-nasi-min
+;;	     max (point-max))
+;;       (skk-annotation-update-jisyo-format-1 min max)))
+;;
+;;   (defun skk-annotation-update-jisyo-format-1 (min max)
+;;     (let (candidate)
+;;       (goto-char min)
+;;       (while (re-search-forward "\\/\\([^\n/]*;[^\n/]*\\)\\/" max t nil)
+;;	 (setq candidate (buffer-substring-no-properties
+;;			  (match-beginning 1) (match-end 1)))
+;;	 (delete-region (match-beginning 1) (match-end 1))
+;;	 (goto-char (match-beginning 1))
+;;	 (insert
+;;	  (concat "(concat \""
+;;		  (mapconcat
+;;		   (function
+;;		    (lambda (c)
+;;		      (if (eq c ?\;)
+;;			  "\\073"
+;;			(char-to-string c))))
+;;		   (append candidate nil) "")
+;;		  "\")")))))
+;;
+;; $B8D?M<-=q$rFI$_$3$_!"<-=q$rFI$_9~$s$@%P%C%U%!$G(B
+;;
+;;   M-x skk-annotation-update-jisyo-format
+;;
+;; $B$9$k$3$H$G$3$N:n6H$r9T$J$&$3$H$,$G$-$^$9!#(B
+;;
+;; $BC"$7!"4{$K%"%N%F!<%7%g%s$,IU$1$i$l$F$$$k>l9g$O!"$3$N%"%N%F!<%7%g%s(B
+;; $B<+BN$b8uJd$H6hJL$G$-$:$K(B quote $B$5$l$F$7$^$$$^$9$N$G!"$4Cm0U2<$5$$(B
+;; ($B:#$N$H$3$m<j:n6H$G(B quote $B$5$l$J$$$h$&$KB`Hr$9$k$J$I$7$+J}K!$O$"$j(B
+;; $B$^$;$s(B)$B!#(B
 
 ;;; Code:
 
@@ -337,7 +374,7 @@
 	       (skk-annotation-show-2 annotation)))
 	    ((eq command browse-command)
 	     (setq list (delq browse-command list))
-	     (setq urls (delq nil (mapcar #'skk-annotation-find-url notes)))
+;	     (setq urls (delq nil (mapcar #'skk-annotation-find-url notes)))
 	     (when word
 	       (cond ((setq cache
 			    (skk-annotation-wikipedia-cache word sources))
@@ -802,35 +839,6 @@ no-previous-annotation $B$r;XDj$9$k$H(B \(C-u M-x skk-annotation-add $B$G;XDj
 	 (unless quiet
 	   (message "%s" "Quoted")))))))
 
-;;;###autoload
-(defun skk-annotation-update-jisyo-format ()
-  (interactive)
-  (skk-setup-jisyo-buffer)
-  (let ((min skk-okuri-ari-min) (max skk-okuri-ari-max))
-    (skk-annotation-update-jisyo-format-1 min max)
-    (setq min skk-okuri-nasi-min
-	  max (point-max))
-    (skk-annotation-update-jisyo-format-1 min max)))
-
-(defun skk-annotation-update-jisyo-format-1 (min max)
-  (let (candidate)
-    (goto-char min)
-    (while (re-search-forward "\\/\\([^\n/]*;[^\n/]*\\)\\/" max t nil)
-      (setq candidate (buffer-substring-no-properties
-		       (match-beginning 1) (match-end 1)))
-      (delete-region (match-beginning 1) (match-end 1))
-      (goto-char (match-beginning 1))
-      (insert
-       (concat "(concat \""
-	       (mapconcat
-		(function
-		 (lambda (c)
-		   (if (eq c ?\;)
-		       "\\073"
-		     (char-to-string c))))
-		(append candidate nil) "")
-	       "\")")))))
-
 ;;;###autoload
 (defun skk-annotation-wikipedia (word &optional sources)
   "Wiktionary/Wikipedia $B$N(B WORD $B$KAjEv$9$k5-;v$+$i%"%N%F!<%7%g%s$r<hF@$9$k!#(B"
@@ -844,18 +852,20 @@ no-previous-annotation $B$r;XDj$9$k$H(B \(C-u M-x skk-annotation-add $B$G;XDj
 	    (while (and (not note)
 			sources)
 	      (setq source (car sources))
+	      ;; Wiktionary $B$G$O$=$N$^$^!"(BWikipedia $B$G$OBh(B 1 $BJ8;z$N$_(B upcase
 	      (setq note (skk-annotation-wikipedia-1 word source
 						     (= 1 (length sources))))
+	      ;;
 	      (when (and (null note)
 			 (memq source '(en.wiktionary ja.wiktionary))
 			 (skk-ascii-char-p (aref word 0))
 			 (not (skk-lower-case-p (aref word 0))))
-		;; Wiktionary $B$O(B downcase $B$,4pK\(B
-		;; (Wikipedia $B$O(B initial $B$,(B upcase)
+		;; Wiktionary $B$G$9$Y$F(B downcase $B$9$k>l9g(B
 		(setq note (skk-annotation-wikipedia-1
 			    (downcase word)
 			    source
 			    (= 1 (length sources)))))
+	      ;;
 	      (setq string (format (if (string= "" string)
 				       "%s%s"
 				     "%s/%s")
@@ -904,7 +914,7 @@ no-previous-annotation $B$r;XDj$9$k$H(B \(C-u M-x skk-annotation-add $B$G;XDj
 	 (append '(("sup" . skk-annotation-wikipedia-clean-sup)
 		   ("sub" . skk-annotation-wikipedia-clean-sub))
 		 html2text-format-tag-list))
-	buffer note aimai continue nop point top pt1 pt2 btag etag end)
+	buf buffer note)
     (if (get-buffer cache-buffer)
 	(with-current-buffer cache-buffer
 	  (setq note (buffer-string)))
@@ -914,439 +924,362 @@ no-previous-annotation $B$r;XDj$9$k$H(B \(C-u M-x skk-annotation-add $B$G;XDj
 				  source word)
 				 #'skk-annotation-wikipedia-retrieved
 				 (list (list source))))
-      (when (and (setq buffer
-		       (catch 'skk-annotation-wikipedia-retrieved
-			 (progn
-			   (condition-case nil
-			       (skk-sit-for 100)
-			     (quit
-			      ;; C-g $B$5$l$?$H$-$N5sF0$rD4@0$9$k(B
-			      ;; $B8=>u$G$O$?$@(B retrieval $B$rCf;_$7$F$*$/$,!"(B
-			      ;; $B"'%b!<%I$G$N(B C-g $B$G$O"&%b!<%I$KI|5"$9$k5sF0$b(B
-			      ;; $B$"$j$&$k$H;W$o$l$k!#(B
-			      nil))
-			   (when (buffer-live-p buffer)
-			     (kill-buffer buffer))
-			   (throw 'skk-annotation-wikipedia-suspended
-				  source))))
+      (while (not buf)
+	(setq buf (catch 'skk-annotation-wikipedia-retrieved
+		    (condition-case nil
+			(sleep-for 0.01)
+		      ((error)
+		       (kill-buffer buffer)
+		       (throw 'skk-annotation-wikipedia-suspended
+			      source))))))
+      (when (and (setq buffer buf)
 		 (buffer-live-p buffer))
-	(with-current-buffer buffer
-	  (set-buffer-multibyte t)
-	  (decode-coding-region (point-min) (point-max) 'utf-8)
-	  (when (> (buffer-size) 0)
-	    (when (get-buffer cache-buffer)
-	      (kill-buffer cache-buffer))
-	    (rename-buffer cache-buffer)
-	    ;; $BMW$i$J$$ItJ,$r>C$9(B
-	    (cond
-	     ;; ja.wiktionary
-	     ((eq source 'ja.wiktionary)
-	      (goto-char (point-min))
-	      (if (save-excursion
-		    (re-search-forward "\
+	(skk-annotation-wikipedia-format-buffer source buffer cache-buffer)))))
+
+(defun skk-annotation-wikipedia-format-buffer (source buffer cache-buffer)
+  (let ((html2text-remove-tag-list
+	 (append '("a" "span" "table" "tr" "td" "h2" "h3" "h4" "h5" "small"
+		   "code")
+		 html2text-remove-tag-list))
+	(html2text-format-tag-list
+	 (append '(("sup" . skk-annotation-wikipedia-clean-sup)
+		   ("sub" . skk-annotation-wikipedia-clean-sub))
+		 html2text-format-tag-list))
+	note aimai continue nop point top pt1 pt2 btag etag end)
+    (with-current-buffer buffer
+      (set-buffer-multibyte t)
+      (decode-coding-region (point-min) (point-max) 'utf-8)
+      (when (> (buffer-size) 0)
+	(when (get-buffer cache-buffer)
+	  (kill-buffer cache-buffer))
+	(rename-buffer cache-buffer)
+	;; $BMW$i$J$$ItJ,$r>C$9(B
+	(cond
+	 ;; ja.wiktionary
+	 ((eq source 'ja.wiktionary)
+	  (goto-char (point-min))
+	  (if (save-excursion
+		(re-search-forward "\
 \\(^HTTP/1\\.0 301 Moved Permanently\\|<div class=\"noarticletext\">\
 \\|:Badtitle\\)"
-				       nil t))
-		  ;; $B9`L\$,$J$$>l9g(B
-		  (erase-buffer)
-		(search-forward "<!-- start content -->" nil t)
-		(delete-region (point-min) (point))
-		;;
-		(goto-char (point-min))
-		(when (re-search-forward
-		       skk-annotation-ja-wiktionary-lang-regexp
-		       nil t)
-		  (save-excursion
-		    (goto-char (match-end 2))
-		    (insert ", "))
-		  (delete-region (point-min) (match-beginning 0))
-		  (setq top (point))
-		  (when (re-search-forward
-			 skk-annotation-ja-wiktionary-lang-regexp
-			 nil t)
-		    (delete-region (setq pt1 (match-beginning 0))
-				   (point-max))))
-		;;
-		(setq point top)
-		(goto-char (point-min))
-		;; ja.wiktionary $B$N=q<0$,(B en.wiktionary $B$[$I@0$C$F$$$J$$$N$G(B
-		;; workaround
-		(unless
-		    (save-excursion
-		       (re-search-forward
-			skk-annotation-ja-wiktionary-part-of-speech-regexp
-			nil t))
-		  (setq point pt1))
-		;;
-		(while (re-search-forward
-			skk-annotation-ja-wiktionary-part-of-speech-regexp
-			nil t)
-		  (setq nop t)
-		  (save-match-data
-		    (when (looking-at "</h3>")
-		      (delete-region (match-beginning 0) (match-end 0))))
-		  (goto-char (match-beginning 0))
-		  (delete-region (or point (point-min)) (point))
-		  (when (re-search-forward "<\\(ol\\|dl\\)>" nil t)
-		    (setq btag (match-string 0)
-			  etag (if (string= btag "<ol>")
-				   "</ol>"
-				 "</dl>")
-			  point nil
-			  pt1 (point)
-			  pt2 nil)
-		    (while (and (not point)
-				(search-forward etag nil t))
-		      (setq pt2 (point))
-		      (goto-char pt1)
-		      (if (and (search-forward btag nil t)
-			       (< (point) pt2))
-			  (progn
-			    (goto-char pt2)
-			    (setq pt1 (point)))
-			(setq point pt2)
-			(goto-char point)))))
-		;;
-		;; ja.wiktionary $B$N=q<0$,(B en.wiktionary $B$[$I@0$C$F$$$J$$$N$G(B
-		;; $B>C$7$9$.$F$7$^$&4m81@-$"$j!#(B
-		(when point
-		  (delete-region point (point-max)))
-		;; ($BMQNc$J$I$r=|$/(B -- $B=|$+$J$$$[$&$,$$$$!)(B)
-		;; ja.wiktionary $B$O(B en.wiktionary $B$HA4$/E}0l$5$l$?=q$-J}$K$O(B
-		;; $B$J$C$F$$$J$$$N$G!"(Bul $B$r=|$/$H>pJs$,$[$H$s$I;D$i$J$$>l9g$,(B
-		;; $B$"$k(B
-		;; (skk-annotation-wikipedia-remove-nested "<ul>" "</ul>")
-		(skk-annotation-wikipedia-remove-nested "<dl>" "</dl>")
-		(skk-annotation-wikipedia-remove-nested "<table[^<]*>"
-							"</table>")
-		(skk-annotation-wikipedia-remove-nested "\
+				   nil t))
+	      ;; $B9`L\$,$J$$>l9g(B
+	      (erase-buffer)
+	    (search-forward "<!-- start content -->" nil t)
+	    (delete-region (point-min) (point))
+	    ;;
+	    (goto-char (point-min))
+	    (when (re-search-forward
+		   skk-annotation-ja-wiktionary-lang-regexp
+		   nil t)
+	      (save-excursion
+		(goto-char (match-end 2))
+		(insert ", "))
+	      (delete-region (point-min) (match-beginning 0))
+	      (setq top (point))
+	      (when (re-search-forward
+		     skk-annotation-ja-wiktionary-lang-regexp
+		     nil t)
+		(delete-region (setq pt1 (match-beginning 0))
+			       (point-max))))
+	    ;;
+	    (setq point top)
+	    (goto-char (point-min))
+	    ;; ja.wiktionary $B$N=q<0$,(B en.wiktionary $B$[$I@0$C$F$$$J$$$N$G(B
+	    ;; workaround
+	    (unless
+		(save-excursion
+		  (re-search-forward
+		   skk-annotation-ja-wiktionary-part-of-speech-regexp
+		   nil t))
+	      (setq point pt1))
+	    ;;
+	    (while (re-search-forward
+		    skk-annotation-ja-wiktionary-part-of-speech-regexp
+		    nil t)
+	      (setq nop t)
+	      (save-match-data
+		(when (looking-at "</h3>")
+		  (delete-region (match-beginning 0) (match-end 0))))
+	      (goto-char (match-beginning 0))
+	      (delete-region (or point (point-min)) (point))
+	      (when (re-search-forward "<\\(ol\\|dl\\)>" nil t)
+		(setq btag (match-string 0)
+		      etag (if (string= btag "<ol>")
+			       "</ol>"
+			     "</dl>")
+		      point nil
+		      pt1 (point)
+		      pt2 nil)
+		(while (and (not point)
+			    (search-forward etag nil t))
+		  (setq pt2 (point))
+		  (goto-char pt1)
+		  (if (and (search-forward btag nil t)
+			   (< (point) pt2))
+		      (progn
+			(goto-char pt2)
+			(setq pt1 (point)))
+		    (setq point pt2)
+		    (goto-char point)))))
+	    ;;
+	    ;; ja.wiktionary $B$N=q<0$,(B en.wiktionary $B$[$I@0$C$F$$$J$$$N$G(B
+	    ;; $B>C$7$9$.$F$7$^$&4m81@-$"$j!#(B
+	    (when point
+	      (delete-region point (point-max)))
+	    ;; ($BMQNc$J$I$r=|$/(B -- $B=|$+$J$$$[$&$,$$$$!)(B)
+	    ;; ja.wiktionary $B$O(B en.wiktionary $B$HA4$/E}0l$5$l$?=q$-J}$K$O(B
+	    ;; $B$J$C$F$$$J$$$N$G!"(Bul $B$r=|$/$H>pJs$,$[$H$s$I;D$i$J$$>l9g$,(B
+	    ;; $B$"$k(B
+	    ;; (skk-annotation-wikipedia-remove-nested "<ul>" "</ul>")
+	    (skk-annotation-wikipedia-remove-nested "<dl>" "</dl>")
+	    (skk-annotation-wikipedia-remove-nested "<table[^<]*>"
+						    "</table>")
+	    (skk-annotation-wikipedia-remove-nested "\
 <div class=\"\\(infl-table\\|thumb.+\\)\"[^<]*>" "</div>" "<div[^<]*>")
-		;;
-		(goto-char (point-min))
-		(while (re-search-forward
-			"<span.*>\\[<a.+>$BJT=8(B</a>\\]</span>"
-			nil t)
-		  (replace-match ""))))
-	     ;; en.wiktionary
-	     ((eq source 'en.wiktionary)
-	      (goto-char (point-min))
-	      (if (save-excursion
-		    (re-search-forward "\
+	    ;;
+	    (goto-char (point-min))
+	    (while (re-search-forward
+		    "<span.*>\\[<a.+>$BJT=8(B</a>\\]</span>"
+		    nil t)
+	      (replace-match ""))))
+	 ;; en.wiktionary
+	 ((eq source 'en.wiktionary)
+	  (goto-char (point-min))
+	  (if (save-excursion
+		(re-search-forward "\
 \\(^HTTP/1\\.0 301 Moved Permanently\\|<div class=\"noarticletext\">\
 \\|:Badtitle\\)"
-				       nil t))
-		  ;; $B9`L\$,$J$$>l9g(B
-		  (erase-buffer)
-		(search-forward "<!-- start content -->" nil t)
-		(delete-region (point-min) (point))
-		;;
-		(goto-char (point-min))
-		(when (re-search-forward
-		       skk-annotation-en-wiktionary-lang-regexp
-		       nil t)
-		  (save-excursion
-		    (goto-char (match-end 2))
-		    (insert ", "))
-		  (delete-region (point-min) (match-beginning 0))
-		  (setq top (point))
-		  (when (re-search-forward
-			 skk-annotation-en-wiktionary-lang-regexp
-			 nil t)
-		    (delete-region (match-beginning 0) (point-max))))
-		;;
-		(setq point top)
-		(goto-char (point-min))
-		(while (re-search-forward
-			 skk-annotation-en-wiktionary-part-of-speech-regexp
-			nil t)
-		  (setq nop t)
-		  (save-match-data
-		    (when (looking-at "</h3>")
-		      (delete-region (match-beginning 0) (match-end 0))))
-		  (goto-char (match-beginning 0))
-		  (delete-region (or point (point-min)) (point))
-		  (when (re-search-forward "<\\(ol\\|dl\\)>" nil t)
-		    (setq btag (match-string 0)
-			  etag (if (string= btag "<ol>")
-				   "</ol>"
-				 "</dl>")
-			  point nil
-			  pt1 (point)
-			  pt2 nil)
-		    (while (and (not point)
-				(search-forward etag nil t))
-		      (setq pt2 (point))
-		      (goto-char pt1)
-		      (if (and (search-forward btag nil t)
-			       (< (point) pt2))
-			  (progn
-			    (goto-char pt2)
-			    (setq pt1 (point)))
-			(setq point pt2)
-			(goto-char point)))))
-		;;
-		(when point
-		  (delete-region point (point-max)))
-		;; ($BMQNc$J$I$r=|$/(B -- $B=|$+$J$$$[$&$,$$$$!)(B)
-		(skk-annotation-wikipedia-remove-nested "<ul>" "</ul>")
-		(skk-annotation-wikipedia-remove-nested "<dl>" "</dl>")
-		(skk-annotation-wikipedia-remove-nested "<table[^<]*>"
-							"</table>")
-		(skk-annotation-wikipedia-remove-nested "\
+				   nil t))
+	      ;; $B9`L\$,$J$$>l9g(B
+	      (erase-buffer)
+	    (search-forward "<!-- start content -->" nil t)
+	    (delete-region (point-min) (point))
+	    ;;
+	    (goto-char (point-min))
+	    (when (re-search-forward
+		   skk-annotation-en-wiktionary-lang-regexp
+		   nil t)
+	      (save-excursion
+		(goto-char (match-end 2))
+		(insert ", "))
+	      (delete-region (point-min) (match-beginning 0))
+	      (setq top (point))
+	      (when (re-search-forward
+		     skk-annotation-en-wiktionary-lang-regexp
+		     nil t)
+		(delete-region (match-beginning 0) (point-max))))
+	    ;;
+	    (setq point top)
+	    (goto-char (point-min))
+	    (while (re-search-forward
+		    skk-annotation-en-wiktionary-part-of-speech-regexp
+		    nil t)
+	      (setq nop t)
+	      (save-match-data
+		(when (looking-at "</h3>")
+		  (delete-region (match-beginning 0) (match-end 0))))
+	      (goto-char (match-beginning 0))
+	      (delete-region (or point (point-min)) (point))
+	      (cond
+	       ((re-search-forward "<\\(ol\\|dl\\)>" nil t)
+		(setq btag (match-string 0)
+		      etag (if (string= btag "<ol>")
+			       "</ol>"
+			     "</dl>")
+		      point nil
+		      pt1 (point)
+		      pt2 nil)
+		(while (and (not point)
+			    (search-forward etag nil t))
+		  (setq pt2 (point))
+		  (goto-char pt1)
+		  (if (and (search-forward btag nil t)
+			   (< (point) pt2))
+		      (progn
+			(goto-char pt2)
+			(setq pt1 (point)))
+		    (setq point pt2)
+		    (goto-char point))))
+	       (t
+		(goto-char (match-end 0))
+		(when (search-forward "</p>" nil t)
+		  (setq point (point))))))
+	    ;;
+	    (when point
+	      (delete-region point (point-max)))
+	    ;; ($BMQNc$J$I$r=|$/(B -- $B=|$+$J$$$[$&$,$$$$!)(B)
+	    (skk-annotation-wikipedia-remove-nested "<ul>" "</ul>")
+	    (skk-annotation-wikipedia-remove-nested "<dl>" "</dl>")
+	    (skk-annotation-wikipedia-remove-nested "<table[^<]*>"
+						    "</table>")
+	    (skk-annotation-wikipedia-remove-nested "\
 <div class=\"\\(infl-table\\|thumb.+\\)\"[^<]*>" "</div>" "<div[^<]*>")
-		(skk-annotation-wikipedia-remove-nested "\
+	    (skk-annotation-wikipedia-remove-nested "\
 <span class=\"interProject\">" "</span>")
-		;; Wikipedia $B$X$N0FFb$r=|$/(B
-		(goto-char (point-min))
-		(while (re-search-forward "\
+	    ;; Wikipedia $B$X$N0FFb$r=|$/(B
+	    (goto-char (point-min))
+	    (while (re-search-forward "\
 \\(<a href=\"/wiki/Wikipedia\" title=\"Wikipedia\">\\)?\
 Wikipedia\\(</a>\\)? has an article on:$" nil t)
-		  (save-excursion
-		    (goto-char (match-beginning 0))
-		    (beginning-of-line)
-		    (setq point (point)))
-		  (forward-line 2)
-		  (end-of-line)
-		  (delete-region point (point)))
-		;;
-		(goto-char (point-min))
-		(while (re-search-forward "\
-<span.*>\\(\\[<a.+>edit</a>\\]\\|Inflection\\)</span>"
-			nil t)
-		  (replace-match ""))))
-	     ;; ja.wikipedia
-	     ((eq source 'ja.wikipedia)
-	      (goto-char (point-min))
-	      (if (save-excursion
-		    (re-search-forward "\
-\\(^HTTP/1\\.0 301 Moved Permanently\\|<div class=\"noarticletext\">\
-\\|:Badtitle\\)"
-				       nil t))
-		  ;; $B9`L\$,$J$$>l9g(B
-		  (erase-buffer)
-		(setq aimai
-		      (save-excursion
-			(search-forward "<a href=\"/wiki/Wikipedia:\
-%E6%9B%96%E6%98%A7%E3%81%95%E5%9B%9E%E9%81%BF\"" nil t)))
-		(search-forward "<!-- start content -->" nil t)
-		(delete-region (point-min) (point))
-		;; <div> $B$r=|5n$9$k(B
-		(skk-annotation-wikipedia-remove-nested "<div.+>" "</div>")
-		;; <span> $B$r=|5n$9$k(B
-		(setq point nil)
-		(goto-char (point-min))
-		(while (re-search-forward
-			"<span class=\"\\(.+audiolink.+\\)\".*>" nil t)
-		  (setq point (match-beginning 0))
-		  (goto-char point)
-		  (search-forward "</span>" nil t)
-		  (delete-region point (point))
-		  (goto-char point))
-		;; <big> $B$r=|5n$9$k(B
-		(goto-char (point-min))
-		(while (re-search-forward "<p><big>.+</big></p>" nil t)
-		  (replace-match ""))
-		;; &#160; $B$r=hM}(B
-		(goto-char (point-min))
-		(while (re-search-forward "&#160;" nil t)
-		  (replace-match " "))
-		;; <br /> $B$r=|5n$9$k(B
-		(goto-char (point-min))
-		(while (re-search-forward "<p>.+\\(<br />\\)$" nil t)
-		  (replace-match "" nil nil nil 1))
-		;; xxx > xxx > xxx ... $B$r=|5n$9$k(B
-		(goto-char (point-min))
-		(while (re-search-forward
-			"<p>.+</a> &gt; \\(<a.+>\\|<b>\\).+</p>" nil t)
-		  (replace-match ""))
-		;; <table> $B$r=|5n(B
-		(skk-annotation-wikipedia-remove-nested "<table.*>" "</table>")
-		;;
-		(goto-char (point-min))
-		(when (or (when (re-search-forward
-				 "<p>\\(<br />\n\\|[^\n]*\\)?\
-<b>[^\n]+</b>[^\n]+"
-				 nil t)
-			    (goto-char (match-beginning 0))
-			    (if (and (save-excursion
-				       (re-search-forward "</p>" nil t))
-				     (string-match "$B!#(B\\|$B!%(B"
-						   (buffer-substring
-						    (point)
-						    (match-beginning 0))))
-				t
-			      (setq point (point)
-				    continue t)
-			      nil))
-			  (when (progn
-				  (goto-char (point-min))
-				  (re-search-forward "<\\(u\\|o\\)l>" nil t))
-			    (goto-char (if continue
-					   point
-					 (match-beginning 0)))
-			    (setq nop t)))
-		  (delete-region (point-min) (point))
-		  (goto-char (point-min))
-		  ;;
-		  (cond
-		   ((or aimai nop)
-		    (setq pt1 (if (re-search-forward "<\\(u\\|o\\)l>" nil t)
-				  (match-end 0)
-				nil)
-			  pt2 nil)
-		    (while (and (not end)
-				(re-search-forward "</\\(u\\|o\\)l>"
-						   nil t))
-		      (setq pt2 (match-end 0))
-		      (save-excursion
-			(goto-char (or pt1 (1+ (point-min))))
-			(when (re-search-forward "<\\(u\\|o\\)l>"
-						 nil t)
-			  (setq pt1 (match-end 0))))
-		      (when (or (null pt1)
-				(> pt1 pt2))
-			(setq end t))))
-		   (t
-		    (re-search-forward "</p>" nil t)))
-		  (delete-region (point) (point-max)))))
-	     ((memq source '(simple.wikipedia en.wikipedia))
-	      (goto-char (point-min))
-	      (if (save-excursion
-		    (re-search-forward "\
-\\(^HTTP/1\\.0 301 Moved Permanently\\|<div class=\"noarticletext\">\
-\\|:Badtitle\\)"
-				       nil t))
-		  ;; $B9`L\$,$J$$>l9g(B
-		  (erase-buffer)
-		(setq aimai
-		      (save-excursion
-			(search-forward "<a href=\"/wiki/Wikipedia:\
-Disambiguation\"" nil t)))
-		(search-forward "<!-- start content -->" nil t)
-		(delete-region (point-min) (point))
-		;; <div> $B$r=|5n$9$k(B
-		(skk-annotation-wikipedia-remove-nested "<div.+>" "</div>")
-		;; <span> $B$r=|5n$9$k(B
-		(setq point nil)
-		(goto-char (point-min))
-		(while (re-search-forward
-			"<span class=\"\\(.+audiolink.+\\)\".*>" nil t)
-		  (setq point (match-beginning 0))
-		  (goto-char point)
-		  (search-forward "</span>" nil t)
-		  (delete-region point (point))
-		  (goto-char point))
-		;; <big> $B$r=|5n$9$k(B
-		(goto-char (point-min))
-		(while (re-search-forward "<p><big>.+</big></p>" nil t)
-		  (replace-match ""))
-		;; &#160; $B$r=hM}(B
-		(goto-char (point-min))
-		(while (re-search-forward "&#160;" nil t)
-		  (replace-match " "))
-		;; <br /> $B$r=|5n$9$k(B
-		(goto-char (point-min))
-		(while (re-search-forward "<p>.+\\(<br />\\)$" nil t)
-		  (replace-match "" nil nil nil 1))
-		;; xxx > xxx > xxx ... $B$r=|5n$9$k(B
-		(goto-char (point-min))
-		(while (re-search-forward
-			"<p>.+</a> &gt; \\(<a.+>\\|<b>\\).+</p>" nil t)
-		  (replace-match ""))
-		;; <table> $B$r=|5n(B
-		(skk-annotation-wikipedia-remove-nested "<table.*>"
-							"</table>")
-		;;
-		(goto-char (point-min))
-		(when (or (when (re-search-forward
-				 "<p>\\(<br />\n\\|[^\n]*\\)?\
-<b>[^\n]+</b>[^\n]+"
-				 nil t)
-			    (goto-char (match-beginning 0))
-			    (if (and (save-excursion
-				       (re-search-forward "</p>" nil t))
-				     (string-match "\\."
-						   (buffer-substring
-						    (point)
-						    (match-beginning 0))))
-				t
-			      (setq point (point)
-				    continue t)
-			      nil))
-			  (when (progn
-				  (goto-char (point-min))
-				  (re-search-forward "<\\(u\\|o\\)l>" nil t))
-			    (goto-char (if continue
-					   point
-					 (match-beginning 0)))
-			    (setq nop t)))
-		  (delete-region (point-min) (point))
-		  (goto-char (point-min))
-		  ;;
-		  (cond
-		   ((or aimai nop)
-		    (setq pt1 (if (re-search-forward "<\\(u\\|o\\)l>" nil t)
-				  (match-end 0)
-				nil)
-			  pt2 nil)
-		    (while (and (not end)
-				(re-search-forward "</\\(u\\|o\\)l>"
-						   nil t))
-		      (setq pt2 (match-end 0))
-		      (save-excursion
-			(goto-char (or pt1 (1+ (point-min))))
-			(when (re-search-forward "<\\(u\\|o\\)l>"
-						 nil t)
-			  (setq pt1 (match-end 0))))
-		      (when (or (null pt1)
-				(> pt1 pt2))
-			(setq end t))))
-		   (t
-		    (re-search-forward "</p>" nil t)))
-		  (delete-region (point) (point-max))))))
+	      (save-excursion
+		(goto-char (match-beginning 0))
+		(beginning-of-line)
+		(setq point (point)))
+	      (forward-line 2)
+	      (end-of-line)
+	      (delete-region point (point)))
 	    ;;
+	    (goto-char (point-min))
+	    (while (re-search-forward "\
+<span.*>\\(\\[<a.+>edit</a>\\]\\|Inflection\\)</span>"
+				      nil t)
+	      (replace-match ""))))
+	 ;; wikipedia
+	 ((memq source '(ja.wikipedia simple.wikipedia en.wikipedia))
+	  (goto-char (point-min))
+	  (if (save-excursion
+		(re-search-forward "\
+\\(^HTTP/1\\.0 301 Moved Permanently\\|<div class=\"noarticletext\">\
+\\|:Badtitle\\)"
+				   nil t))
+	      ;; $B9`L\$,$J$$>l9g(B
+	      (erase-buffer)
+	    (setq aimai
+		  (save-excursion
+		    (re-search-forward "<a href=\"/wiki/Wikipedia:\
+\\(%E6%9B%96%E6%98%A7%E3%81%95%E5%9B%9E%E9%81%BF\\|Disambiguation\\)\""
+				       nil t)))
+	    (search-forward "<!-- start content -->" nil t)
+	    (delete-region (point-min) (point))
+	    ;; <div> $B$r=|5n$9$k(B
+	    (skk-annotation-wikipedia-remove-nested "<div.+>" "</div>")
+	    ;; <span> $B$r=|5n$9$k(B
 	    (setq point nil)
-	    (when (> (buffer-size) 0)
-	      (html2text)
+	    (goto-char (point-min))
+	    (while (re-search-forward
+		    "<span class=\"\\(.+audiolink.+\\|editsection\\)\".*>" nil t)
+	      (setq point (match-beginning 0))
+	      (goto-char point)
+	      (search-forward "</span>" nil t)
+	      (delete-region point (point))
+	      (goto-char point))
+	    ;; <big> $B$r=|5n$9$k(B
+	    (goto-char (point-min))
+	    (while (re-search-forward "<p><big>.+</big></p>" nil t)
+	      (replace-match ""))
+	    ;; &#160; $B$r=hM}(B
+	    (goto-char (point-min))
+	    (while (re-search-forward "&#160;" nil t)
+	      (replace-match " "))
+	    ;; <br /> $B$r=|5n$9$k(B
+	    (goto-char (point-min))
+	    (while (re-search-forward "<p>.+\\(<br />\\)$" nil t)
+	      (replace-match "" nil nil nil 1))
+	    ;; xxx > xxx > xxx ... $B$r=|5n$9$k(B
+	    (goto-char (point-min))
+	    (while (re-search-forward
+		    "<p>.+</a> &gt; \\(<a.+>\\|<b>\\).+</p>" nil t)
+	      (replace-match ""))
+	    ;; <script> $B$r=|5n(B
+	    (skk-annotation-wikipedia-remove-nested "<script.*>" "</script>")
+	    ;; <table> $B$r=|5n(B
+	    (skk-annotation-wikipedia-remove-nested "<table.*>" "</table>")
+	    ;;
+	    (goto-char (point-min))
+	    (when (or (when (re-search-forward
+			     "<p>\\(<br />\n\\|[^\n]*\\)?\
+<b>[^\n]+</b>[^\n]+"
+			     nil t)
+			(goto-char (match-beginning 0))
+			(if (and (save-excursion
+				   (re-search-forward "</p>" nil t))
+				 (string-match
+				  (cond
+				   ((eq source 'ja.wikipedia)
+				    "$B!#(B\\|$B!%(B")
+				   (t
+				    "\\."))
+				  (buffer-substring (point)
+						    (match-beginning 0))))
+			    t
+			  (setq point (point)
+				continue t)
+			  nil))
+		      (when (progn
+			      (goto-char (point-min))
+			      (re-search-forward "<\\(u\\|o\\)l>" nil t))
+			(goto-char (if continue
+				       point
+				     (match-beginning 0)))
+			(setq nop t)))
+	      (delete-region (point-min) (point))
 	      (goto-char (point-min))
+	      ;;
 	      (cond
-	       ((memq source '(ja.wiktionary en.wiktionary))
-		;; wiktionary $B$N@07A7k2L$O6u9T$@$i$1$K$J$k(B...
-		(goto-char (point-min))
-		(while (re-search-forward "\n[\n]+" nil t)
-		  (replace-match "\n"))
-		(goto-char (point-min))
-		(while (not (eobp))
-		  (beginning-of-line)
-		  (setq point (point))
-		  (forward-line 1)
-		  (fill-region point (point))))
+	       ((or aimai nop)
+		(setq pt1 (if (re-search-forward "<\\(u\\|o\\)l>" nil t)
+			      (match-end 0)
+			    nil)
+		      pt2 nil)
+		(while (and (not end)
+			    (re-search-forward "</\\(u\\|o\\)l>"
+					       nil t))
+		  (setq pt2 (match-end 0))
+		  (save-excursion
+		    (goto-char (or pt1 (1+ (point-min))))
+		    (when (re-search-forward "<\\(u\\|o\\)l>"
+					     nil t)
+		      (setq pt1 (match-end 0))))
+		  (when (or (null pt1)
+			    (> pt1 pt2))
+		    (setq end t))))
 	       (t
-		(while (looking-at "^[ \t]*$")
-		  (kill-line 1))
-		(cond ((or aimai nop)
-		       (while (not (eobp))
-			 (beginning-of-line)
-			 (setq point (point))
-			 (forward-line 1)
-			 (fill-region point (point))))
-		      (t
-		       (fill-paragraph nil)))))
-	      ;;
-	      (when aimai
-		(insert (if (eq source 'en.wikipedia)
-			    "\n(Disambiguation)"
-			  "\n($B[#Kf$52sHr$N%Z!<%8(B)")))
-	      ;;
-	      (goto-char (point-max))
-	      (while (and (looking-at "^$")
-			  (not (string= "" (buffer-string))))
-		(delete-char -1))
-	      ;;
-	      (when (and (not (equal (buffer-string) ""))
-			 (not (get-text-property 1 'face)))
-		(put-text-property 1 2 'face 'default))
-	      (setq note (buffer-string)))))))
+		(re-search-forward "</p>" nil t)))
+	      (delete-region (point) (point-max))))))
+	;;
+	(setq point nil)
+	(when (> (buffer-size) 0)
+	  (html2text)
+	  (goto-char (point-min))
+	  (cond
+	   ((memq source '(ja.wiktionary en.wiktionary))
+	    ;; wiktionary $B$N@07A7k2L$O6u9T$@$i$1$K$J$k(B...
+	    (goto-char (point-min))
+	    (while (re-search-forward "\n[\n]+" nil t)
+	      (replace-match "\n"))
+	    (goto-char (point-min))
+	    (while (not (eobp))
+	      (beginning-of-line)
+	      (setq point (point))
+	      (forward-line 1)
+	      (fill-region point (point))))
+	   (t
+	    (while (looking-at "^[ \t]*$")
+	      (kill-line 1))
+	    (cond ((or aimai nop)
+		   (while (not (eobp))
+		     (beginning-of-line)
+		     (setq point (point))
+		     (forward-line 1)
+		     (fill-region point (point))))
+		  (t
+		   (fill-paragraph nil)))))
+	  ;;
+	  (when aimai
+	    (insert (if (eq source 'ja.wikipedia)
+			"\n($B[#Kf$52sHr$N%Z!<%8(B)"
+		      "\n(Disambiguation)")))
+	  ;;
+	  (goto-char (point-max))
+	  (while (and (looking-at "^$")
+		      (not (string= "" (buffer-string))))
+	    (delete-char -1))
+	  ;;
+	  (when (and (not (equal (buffer-string) ""))
+		     (not (get-text-property 1 'face)))
+	    (put-text-property 1 2 'face 'default))
+	  (setq note (buffer-string)))))
     ;;
     (cond ((stringp note)
 	   (if (equal note "")
@@ -1407,9 +1340,7 @@ Disambiguation\"" nil t)))
 	 (ignore-errors
 	   (throw 'skk-annotation-wikipedia-suspended (cadr args))))
 	(t
-	 (condition-case nil
-	     (throw 'skk-annotation-wikipedia-retrieved (current-buffer))
-	   (error (kill-buffer (current-buffer)))))))
+	 (throw 'skk-annotation-wikipedia-retrieved (current-buffer)))))
 
 ;;;###autoload
 (defun skk-annotation-treat-wikipedia (word &optional sources)
@@ -1468,42 +1399,63 @@ Disambiguation\"" nil t)))
 (defun skk-annotation-wikipedia-cache (word &optional sources)
   (let ((sources (or sources skk-annotation-wikipedia-sources))
 	(word (skk-annotation-wikipedia-normalize-word word 'en.wiktionary))
-	(cword (skk-annotation-wikipedia-normalize-word word)))
+	(cword (skk-annotation-wikipedia-normalize-word word))
+	(ccword (skk-annotation-wikipedia-normalize-word word
+							 'upcase-initials)))
     (catch 'found
       (while sources
 	(let* ((source (pop sources))
 	       (ccache-buffer (if (equal word cword)
 				  nil
 				(format " *skk %s %s" source cword)))
+	       (cccache-buffer (if (or (equal word ccword)
+				       (equal cword ccword))
+				   nil
+				 (format " *skk %s %s" source ccword)))
 	       (cache-buffer (format " *skk %s %s" source word))
 	       string)
 	  (setq string
 		(if (and ccache-buffer
 			 (get-buffer ccache-buffer))
-		    ;; Word
+		    ;; Word word
 		    (with-current-buffer (get-buffer ccache-buffer)
 		      (buffer-string))
 		  ""))
 	  (if (> (length string) 0)
 	      (throw 'found (cons string ccache-buffer))
 	    (setq string
-		  (if (get-buffer cache-buffer)
-		      ;; word
-		      (with-current-buffer (get-buffer cache-buffer)
+		  (if (and cccache-buffer
+			 (get-buffer cccache-buffer))
+		      ;; Word Word
+		      (with-current-buffer (get-buffer cccache-buffer)
 			(buffer-string))
 		    ""))
-	    (if (string= string "")
-		nil
-	      (throw 'found (cons string cache-buffer)))))))))
+	    (if (> (length string) 0)
+		(throw 'found (cons string cccache-buffer))
+	      (setq string
+		    (if (get-buffer cache-buffer)
+			;; word word
+			(with-current-buffer (get-buffer cache-buffer)
+			  (buffer-string))
+		      ""))
+	      (if (string= string "")
+		  nil
+		(throw 'found (cons string cache-buffer))))))))))
 
 ;;;###autoload
 (defun skk-annotation-wikipedia-region (&optional prefix-arg start end)
   (interactive (cons (prefix-numeric-value current-prefix-arg)
-		     (if (static-if (featurep 'xemacs)
-			     (region-active-p)
-			   (and transient-mark-mode mark-active))
-			 (list (region-beginning) (region-end))
-		       (list 1 1))))
+		     (cond
+		      ((static-if (featurep 'xemacs)
+			   (region-active-p)
+			 (and transient-mark-mode mark-active))
+		       (list (region-beginning) (region-end)))
+		      ((eq skk-henkan-mode 'on)
+		       (list (marker-position skk-henkan-start-point)
+			     (point)))
+		      (t
+		       ;; dummy
+		       (list 1 1)))))
   ;; $B%_%K%P%C%U%!$K$$$k$H$-M>7W$J%a%C%;!<%8$r%/%j%"$9$k(B
   (when (or skk-isearch-switch
 	    (skk-in-minibuffer-p))
@@ -1541,23 +1493,26 @@ Disambiguation\"" nil t)))
 		     args))
     (error "%s" "URL $B%Q%C%1!<%8$^$?$O(B Mule-UCS $B$,MxMQ$G$-$^$;$s(B")))
 
-(defun skk-annotation-wikipedia-normalize-word (word &optional source)
+(defun skk-annotation-wikipedia-normalize-word (word &optional method)
   ;; $B%9%Z!<%9$O(B %20 $B$G$O$J$/!"%"%s%@!<%9%3%"$KJQ49$9$k(B
   (replace-regexp-in-string " "
 			    "_"
-			    (if (memq source '(ja.wiktionary en.wiktionary))
-				(if (and (> (length word) 1)
-					 (skk-ascii-char-p (aref word 0))
-					 (skk-lower-case-p (aref word 1)))
-				    ;; $BFsJ8;z$a$,(B lower case $B$J$i(B downcase
-				    (downcase word)
-				  ;; $B0lJ8;z$@$C$?$i85$N(B case
-				  ;; $BFsJ8;z$a$,(B upper case $B$J$i85$N(B case
-				  ;; $B1Q8l0J30$OL$BP1~(B
-				  word)
-			      ;(upcase-initials word)
-			      (concat (vector (upcase (aref word 0)))
-				      (substring word 1)))))
+			    (cond
+			     ((memq method '(ja.wiktionary en.wiktionary))
+			      (if (and (> (length word) 1)
+				       (skk-ascii-char-p (aref word 0))
+				       (skk-lower-case-p (aref word 1)))
+				  ;; $BFsJ8;z$a$,(B lower case $B$J$i(B downcase
+				  (downcase word)
+				;; $B0lJ8;z$@$C$?$i85$N(B case
+				;; $BFsJ8;z$a$,(B upper case $B$J$i85$N(B case
+				;; $B1Q8l0J30$OL$BP1~(B
+				word))
+			      ((eq method 'upcase-initials)
+			       (upcase-initials word))
+			      (t
+			       (concat (vector (upcase (aref word 0)))
+				       (substring word 1))))))
 
 (defun skk-annotation-url-package-available-p ()
   (when (eq skk-annotation-url-package-available-p 'untested)
