@@ -6,9 +6,9 @@
 
 ;; Author: Masahiko Sato <masahiko@kuis.kyoto-u.ac.jp>
 ;; Maintainer: SKK Development Team <skk@ring.gr.jp>
-;; Version: $Id: skk-comp.el,v 1.71 2007/04/27 23:22:49 skk-cvs Exp $
+;; Version: $Id: skk-comp.el,v 1.79 2009/07/10 23:19:00 skk-cvs Exp $
 ;; Keywords: japanese, mule, input method
-;; Last Modified: $Date: 2007/04/27 23:22:49 $
+;; Last Modified: $Date: 2009/07/10 23:19:00 $
 
 ;; This file is part of Daredevil SKK.
 
@@ -71,6 +71,7 @@
 	;; buffer-local $BCM$r2u$962$l$"$j!#(B
 	skk-num-list
 	tmp-key data
+	orig-key
 	c-word)
     (when first
       (setq skk-comp-search-done nil
@@ -104,8 +105,13 @@
 	(unless (and skk-comp-use-prefix
 		     (eq data t))
 	  (setq skk-comp-prefix ""))))
+    (setq orig-key (if skk-katakana
+		       (skk-hiragana-to-katakana skk-comp-key)
+		     skk-comp-key))
+    (when skk-katakana
+      (setq skk-comp-key (skk-katakana-to-hiragana skk-comp-key)))
     (cond
-     ;; ($B2a5n$KC5:w:Q$_$NFI$_$r%"%/%;%9Cf(B)
+     ;; ($BA48uJdC5:w:Q$_(B)
      (skk-comp-search-done
       (if (= skk-comp-depth 0)
 	  ;; circulate $B$J$i$P(B c-word = skk-comp-key $B$J$N$G(B c-word = nil
@@ -114,19 +120,26 @@
 	      (setq skk-comp-depth (length skk-comp-stack)))
 	(setq skk-comp-depth (1- skk-comp-depth))
 	(setq c-word (nth skk-comp-depth skk-comp-stack))))
-     ;; ($B?75,$NFI$_$rC5:w(B)
+     ;; ($BL$C5:w8uJd$,;D$C$F$$$k2DG=@-M-$j(B)
      (t
-      (setq c-word
-	    (let ((word (skk-comp-get-candidate first)))
-	      (while (member word skk-comp-stack)
-		(setq word (skk-comp-get-candidate)))
-	      word))
-      (if c-word
-	  ;; $B?75,$K8+$D$1$?$H$-$@$1(B push $B$9$k!#(B
-	  (push c-word skk-comp-stack)
-	(setq skk-comp-search-done t)
-	(if skk-comp-circulate
-	    (setq skk-comp-depth (length skk-comp-stack))))))
+      (cond
+       ;; $B:G8e$KF@$i$l$?8uJd$rI=<($7$F$$$k(B
+       ((= skk-comp-depth 0)
+	(setq c-word
+	      (let ((word (skk-comp-get-candidate first)))
+		(while (member word skk-comp-stack)
+		  (setq word (skk-comp-get-candidate)))
+		word))
+	(if c-word
+	    ;; $B?75,$K8+$D$1$?$H$-$@$1(B push $B$9$k!#(B
+	    (push c-word skk-comp-stack)
+	  (setq skk-comp-search-done t)
+	  (if skk-comp-circulate
+	      (setq skk-comp-depth (length skk-comp-stack)))))
+       (t
+	;; "," $B$J$I$GA08uJd$KLa$C$F$$$k(B
+	(setq skk-comp-depth (1- skk-comp-depth)
+	      c-word (nth skk-comp-depth skk-comp-stack))))))
     (cond
      (c-word
       (delete-region skk-henkan-start-point (point))
@@ -135,12 +148,12 @@
       ;; When skk-comp-circulate, return to the keyword.
       (when skk-comp-circulate
 	(delete-region skk-henkan-start-point (point))
-	(insert skk-comp-key))
+	(insert orig-key))
       (unless silent
 	(ding)
 	(cond
 	 ((and (string= skk-comp-key "")
-	       (assq 'skk-comp-by-history skk-current-completion-prog-list)
+	       (assq 'skk-comp-by-history skk-completion-prog-list)
 	       (or (not skk-comp-use-prefix)
 		   (string= skk-comp-prefix "")))
 	  (skk-message "$B$3$l0J>e$NMzNr$O$"$j$^$;$s(B"
@@ -149,14 +162,14 @@
 	  (if skk-japanese-message-and-error
 	      (message "\"%s\" $B$GJd40$9$Y$-8+=P$78l$O(B%s$B$"$j$^$;$s(B"
 		       (if skk-comp-use-prefix
-			   (concat skk-comp-key skk-comp-prefix)
-			 skk-comp-key)
+			   (concat orig-key skk-comp-prefix)
+			 orig-key)
 		       (if first "" "$BB>$K(B"))
 	    (message "No %scompletions for \"%s\""
 		     (if first "" "more ")
 		     (if skk-comp-use-prefix
-			 (concat skk-comp-key skk-comp-prefix)
-		       skk-comp-key))))))))))
+			 (concat orig-key skk-comp-prefix)
+		       orig-key))))))))))
 
 (defun skk-comp-get-candidate (&optional first)
   (when first
@@ -182,7 +195,9 @@
 	(setq skk-current-completion-prog-list
 	      (cdr skk-current-completion-prog-list))
 	(setq skk-comp-first t)))
-    cand))
+    (if (and skk-katakana cand)
+	(skk-hiragana-to-katakana cand)
+      cand)))
 
 ;; for test or backend use
 ;;;###autoload
@@ -199,7 +214,9 @@
       (while (setq cand (skk-comp-get-candidate))
 	(unless (member cand ret)
 	  (setq ret (cons cand ret)))))
-    (nreverse ret)))
+    (nreverse (if skk-katakana
+		  (mapcar 'skk-hiragana-to-katakana ret)
+		ret))))
 
 (defun skk-comp-get-regexp (prefix)
   ;; $B%W%l%U%#%/%9$KBP1~$9$k@55,I=8=$rJV$9!#(B
@@ -397,6 +414,7 @@
 	      (nreverse list))))
     (pop skk-comp-kakutei-midasi-list)))
 
+;;;###autoload
 (defun skk-comp-smart-find (&optional path)
   "`smart-find' $B$,8+$D$1$?%U%!%$%kL>$GJd40$9$k(B"
   (cond (skk-abbrev-mode
@@ -409,6 +427,7 @@
 	(t
 	 (setq skk-comp-smart-find-files nil))))
 
+;;;###autoload
 (defun skk-search-smart-find (&optional path not-abbrev-only
 					without-char-maybe)
   "`smart-find'$B$rMxMQ$7$?JQ49$r9T$J$&!#(B
@@ -425,9 +444,10 @@ NOT-ABBREV-ONLY $B$r;XDj$9$k;v$G>o$KM-8z$H$J$k!#(B"
 
 (defun skk-smart-find (key &optional path)
   ;; smart-find $B$O(B provide $B$5$l$F$$$J$$(B
-  (let ((dont-bind-my-keys t))
-    (load-library "smart-find"))
-  (message "")
+  (unless (fboundp 'smart-find-file)
+    (let ((dont-bind-my-keys t))
+      (load-library "smart-find"))
+    (message ""))
   ;;
   (unless path
     (setq path skk-smart-find-file-path))
@@ -446,10 +466,11 @@ NOT-ABBREV-ONLY $B$r;XDj$9$k;v$G>o$KM-8z$H$J$k!#(B"
       (setq files
 	    (sort files #'string-lessp)))))
 
+;;;###autoload
 (defun skk-comp-lisp-symbol (&optional predicate)
   "Lisp symbol $BL>$GJd40$9$k!#(B
-PREDICATE $B$K0z?t(B 1 $B8D$N4X?t$r;XDj$9$l$P!"(BPREDICATE $B$r8+$?$9%7%s%\%k(B
-$B$K8B$C$FJd40$9$k!#(BPREDICATE $B$K$O(B `fboundp', `boundpp', `commandp'
+PREDICATE $B$K0z?t(B 1 $B8D$N4X?t$r;XDj$9$l$P!"(BPREDICATE $B$rK~$?$9%7%s%\%k(B
+$B$K8B$C$FJd40$9$k!#(BPREDICATE $B$K$O(B `fboundp', `boundp', `commandp'
 $B$J$I$,;XDj$G$-$k!#;XDj$7$J$1$l$P4X?t$^$?$OJQ?t$K8B$C$FJd40$9$k!#(B"
   (cond (skk-abbrev-mode
 	 (when skk-comp-first
@@ -471,11 +492,12 @@ PREDICATE $B$K0z?t(B 1 $B8D$N4X?t$r;XDj$9$l$P!"(BPREDICATE $B$r8+$?$9%7%s%\
 	(t
 	 (setq skk-comp-lisp-symbols nil))))
 
+;;;###autoload
 (defun skk-search-lisp-symbol (&optional predicate not-abbrev-only
 					 without-char-maybe)
-  "Lisp symbol $BL>$GJd40$9$k!#(B
-PREDICATE $B$K0z?t(B 1 $B8D$N4X?t$r;XDj$9$l$P!"(BPREDICATE $B$r8+$?$9%7%s%\%k(B
-$B$K8B$C$FJd40$9$k!#(BPREDICATE $B$K$O(B `fboundp', `boundpp', `commandp'
+  "Lisp symbol $BL>$GJd40$7$?7k2L$r8!:w7k2L$H$7$FJV$9!#(B
+PREDICATE $B$K0z?t(B 1 $B8D$N4X?t$r;XDj$9$l$P!"(BPREDICATE $B$rK~$?$9%7%s%\%k(B
+$B$K8B$C$FJd40$9$k!#(BPREDICATE $B$K$O(B `fboundp', `boundp', `commandp'
 $B$J$I$,;XDj$G$-$k!#;XDj$7$J$1$l$P4X?t$^$?$OJQ?t$K8B$C$FJd40$9$k!#(B
 SKK abbrev $B%b!<%I$K$F!"1QJ8;z(B + skk-completion-search-char (~)$B$G(B
 $BL$40%9%Z%k$r;XDj$7$FJQ49$9$k$H!"Jd408uJd$,JQ498uJd$H$7$F=P8=$9$k!#(B
@@ -490,9 +512,10 @@ NOT-ABBREV-ONLY $B$r;XDj$9$k;v$G>o$KM-8z$H$J$k!#(B"
 
 (defun skk-comp-restrict-by-prefix (comp-prog)
   "$BJd40%W%m%0%i%`$K$h$jF@$i$l$?8uJd$r(B `skk-comp-prefix' $B$G9J$j9~$`!#(B
+`skk-comp-prefix' $B$KBP1~$7$F$$$J$$Jd40%W%m%0%i%`$r<j7Z$KBP1~$5$;$k:]$K;H$&!#(B
 
-  (skk-comp-restrict-by-prefix '(skk-comp-by-server-completion))
-$B$N$h$&$J$b$N$r(B `skk-completion-prog-list' $B$K;XDj$9$k!#(B"
+  (skk-comp-restrict-by-prefix '(your-completion-program))
+$B$N$h$&$J$b$N$r(B `skk-completion-prog-list' $B$NMWAG$K;XDj$9$k!#(B"
   (save-match-data
     (let ((regexp-key (concat "^"
 			      (regexp-quote skk-comp-key)
