@@ -29,15 +29,14 @@
 ;;; Code:
 
 (eval-when-compile
+  (require 'static)
   (require 'skk-macs))
 
 (require 'skk-vars)
 (require 'wid-edit)
+(require 'custom)
 
-;;;###autoload
-(defvar skk-custom-file "~/.skk-cus")
 (defvar skk-custom-params nil)
-;;;###autoload
 (defvar skk-custom-alist nil)
 (defvar skk-custom-buffer-original nil)
 (defvar skk-custom-map
@@ -53,6 +52,10 @@
      (const :tag "$B%+!<%=%k$K?'$r$D$1$k(B" t) "")
     (skk-japanese-message-and-error
      (const :tag "$B%a%C%;!<%8$OF|K\8l$GDLCN$9$k(B" t) "")
+    (skk-verbose
+     (const :tag "$B$U$D$&$h$jB?$/$N%a%C%;!<%8$rI=<($9$k(B" t) "")
+    (skk-show-japanese-menu
+     (const :tag "$BF|K\8l%a%K%e!<$r;HMQ$9$k(B" t) "")
     (skk-show-annotation
      (radio :tag "$BJQ49;~$KCp<a$rI=<($9$k!)(B"
 	    (const :tag "$B>o$KI=<((B" t)
@@ -61,7 +64,13 @@
 	    (const :tag "$B8uJd0lMw$H%_%K%P%C%U%!$G$OHsI=<((B"
 		   (not list minibuf))
 	    (const :tag "$BHsI=<((B" nil))
-     "")))
+     "")
+    (skk-show-inline
+     (const :tag "$B8uJd0lMw$r$=$N%&%$%s%I%&$K%$%s%i%$%sI=<($9$k(B" t) "")
+    (skk-show-tooltip
+     (const :tag "$B8uJd0lMw!&Cm<a$r%D!<%k%F%#%C%W$GI=<($9$k(B" t) "")
+    (skk-show-candidates-always-pop-to-buffer
+     (const :tag "$B8uJd0lMw$rJL%&%$%s%I%&$rMQ0U$7$FI=<($9$k(B" t) "")))
 
 (defconst skk-cus-params-ui
   '((skk-egg-like-newline
@@ -86,19 +95,24 @@
 	     (const :tag "$B<+F0=hM}$9$k(B" auto)
 	     (const :tag "$B%f!<%6$N;X<($K$h$k(B" ask)
 	     (const :tag "$B<+F0=hM}$7$J$$(B" nil))
+     "")
+    (skk-j-mode-function-key-usage
+     (radio :tag "$B$+$J%b!<%I$G%U%!%s%/%7%g%s%-!<$r;H$&!)(B"
+	    (const :tag "$BFC<lJQ49$K3d$jEv$F$k(B" conversion)
+	    (const :tag "$B;H$o$J$$(B" nil))
      "")))
 
 (defconst skk-cus-params-search
   '((skk-use-look
      (const :tag "$BJd40$N;~$K(B look $B%3%^%s%I$r;H$&(B" t) "")
     (skk-auto-okuri-process
-     (const :tag "$BAw$j$J$7JQ49$GAw$j$"$j8uJd$b8!:w$9$k(B" t) "")))
+     (const :tag "$BAw$j$J$7JQ49$GAw$j$"$j8uJd$b8!:w$9$k(B" t) "")
+    (skk-use-numeric-conversion
+     (const :tag "$B?tCMJQ495!G=$r;H$&(B" t) "")))
 
 (defconst skk-cus-params-input
   '((skk-use-jisx0201-input-method
-     (const :tag "$BH>3Q%+%J$rF~NO2DG=$K$9$k(B" t) "")
-    (skk-use-kana-keyboard
-     (const :tag "$B$+$JF~NO$r2DG=$K$9$k(B" t) "")))
+     (const :tag "$BH>3Q%+%J$rF~NO2DG=$K$9$k(B" t) "")))
 
 (defconst skk-cus-params-misc
   '((skk-share-private-jisyo
@@ -108,9 +122,21 @@
     (skk-preload
      (const :tag "SKK $B$r$"$i$+$8$a%m!<%I$7$F=i2s5/F0$r9bB.$K$9$k(B" t) "")))
 
-(defun skk-cus-set ()
-  (dolist (param skk-custom-alist)
-    (set (car param) (cdr param))))
+;; For Emacs 21.
+(defun-maybe custom-quote (sexp)
+  "Quote SEXP if it is not self quoting."
+  (if (or (memq sexp '(t nil))
+	  (keywordp sexp)
+	  (and (listp sexp)
+	       (memq (car sexp) '(lambda)))
+	  (stringp sexp)
+	  (numberp sexp)
+	  (vectorp sexp)
+;;;  	  (and (fboundp 'characterp)
+;;;  	       (characterp sexp))
+	  )
+      sexp
+    (list 'quote sexp)))
 
 (defun skk-custom-mode ()
   (kill-all-local-variables)
@@ -129,10 +155,12 @@
 		 nil)))
 	 params)))
 
+;;;###autoload
 (defun skk-customize-group-skk ()
   (interactive)
   (customize-group 'skk))
 
+;;;###autoload
 (defun skk-customize ()
   (interactive)
   (dolist (param (append skk-cus-params-visual
@@ -258,9 +286,9 @@
 (defun skk-customize-done (&rest args)
   (interactive)
   (setq skk-custom-alist nil)
-  (dolist (params skk-custom-params)
+  (dolist (group skk-custom-params)
     (setq skk-custom-alist (append skk-custom-alist
-				   (widget-value params))))
+				   (widget-value group))))
   (dolist (param (append skk-cus-params-visual
 			 skk-cus-params-ui
 			 skk-cus-params-henkan
@@ -270,44 +298,38 @@
     (unless (assq (car param) skk-custom-alist)
       (push (cons (car param) nil) skk-custom-alist)))
   (skk-cus-set)
-  (skk-cus-save-file)
   (bury-buffer)
   (unless (eq skk-custom-buffer-original (current-buffer))
     (switch-to-buffer skk-custom-buffer-original))
   (skk-adjust-user-option))
 
-(defun skk-cus-save-file ()
-  (with-temp-buffer
-    (insert "(setq skk-custom-alist '"
-	    (prin1-to-string skk-custom-alist)
-	    ")\n")
-    (write-region (point-min) (point-max) skk-custom-file)))
-
 ;;;###autoload
 (defun skk-cus-setup ()
-  (let ((file (expand-file-name skk-custom-file)))
-    (when (file-readable-p file)
-      (load-file file)
-      (skk-cus-set))))
+  ;; obsolete
+  (when (and (boundp 'skk-custom-file)
+	     (ignore-errors
+	       (file-readable-p (symbol-value 'skk-custom-file))))
+    (let* ((old-name (expand-file-name (symbol-value 'skk-custom-file)))
+	   (new-name (concat old-name ".not_used")))
+      (load-file old-name)
+      (copy-file old-name new-name 'ok-if-already-exists)
+      (delete-file old-name))
+    (skk-cus-set)))
 
-(defun skk-cus-update ()
-  (let ((params (append skk-cus-params-visual
-			skk-cus-params-ui
-			skk-cus-params-henkan
-			skk-cus-params-search
-			skk-cus-params-input
-			skk-cus-params-misc))
-	param)
-    (setq skk-custom-alist nil)
-    (while params
-      (setq param (pop params))
-      (setq skk-custom-alist
-	    (cons (cons (car param) (symbol-value (car param)))
-		  skk-custom-alist)))
-    (skk-cus-save-file)))
-
-(defadvice custom-save-variables (after skk-cus-update activate)
-  (skk-cus-update))
+(defun skk-cus-set ()
+  (dolist (param skk-custom-alist)
+    (let ((variable (car param))
+	  (value (cdr param)))
+      (funcall (or (get variable 'custom-set) 'set-default) variable value)
+      (put variable 'saved-value (list (custom-quote value)))
+      (static-when (fboundp 'custom-push-theme)
+	(custom-push-theme 'theme-value variable 'user 'set
+			   (custom-quote value)))
+      (put variable 'customized-value nil)
+      (put variable 'customized-variable-comment nil)))
+  (custom-save-all)
+  ;;
+  (setq skk-custom-alist nil))
 
 (require 'product)
 (product-provide
