@@ -132,16 +132,6 @@
     (list
      (cons 'latin skk-e21-modeline-property))))
 
-(defvar skk-e21-coding-system (cond
-			       ((memq window-system '(w32 nil))
-				nil)
-			       ((and (boundp 'mac-carbon-version-string)
-				     window-system
-				     (find-coding-system 'utf-8))
-				'utf-8)
-			       (t
-				locale-coding-system)))
-
 (defvar skk-e21-modeline-menu nil)
 
 ;; Functions.
@@ -152,6 +142,17 @@
     (setq skk-e21-modeline-menu
 	  (easy-menu-create-menu (car skk-e21-modeline-menu-items)
 				 (cdr skk-e21-modeline-menu-items))))
+  ;;
+  (unless (or (null window-system)
+	      (eq window-system 'w32)
+	      (boundp 'mac-carbon-version-string)
+	      (and (eq window-system 'x)
+		   (>= emacs-major-version 22)
+		   (boundp 'gtk-version-string)
+		   (stringp (symbol-value 'gtk-version-string))
+		   (string< "2.0" gtk-version-string)))
+    (setq skk-show-japanese-menu nil))
+  ;;
   (when skk-show-japanese-menu
     (skk-e21-menu-replace skk-e21-modeline-menu)
     (dolist (map (list skk-j-mode-map skk-latin-mode-map
@@ -280,13 +281,10 @@
 	(format "%s" (key-description keys))
       nil)))
 
-(defun skk-e21-encode-string (str)
-  (if (null skk-e21-coding-system)
-      str
-    (encode-coding-string str skk-e21-coding-system)))
-
 (defun skk-e21-menu-replace (list)
-  (let (cons)
+  (let ((running-ntemacs (and (eq window-system 'w32)
+			      (not (fboundp 'Meadow-version))))
+	cons)
     (while (and list (listp list))
       (cond
        ((and (car-safe list)
@@ -294,10 +292,25 @@
 	(skk-e21-menu-replace (car list)))
        ((and (stringp (car-safe list))
 	     (setq cons (assoc (car list) skk-e21-menu-resource-ja)))
-	(setcar list (skk-e21-encode-string (cdr cons))))
+	(setcar list (if (and running-ntemacs
+			      (member (car list) '("Hiragana" "Katakana"
+				      "Hankaku alphabet" "Zenkaku alphabet")))
+			 ;; NTEmacs $B$G(B Widget $BIU$-%a%K%e!<%"%$%F%`$N(B
+			 ;; $BF|K\8l$,$&$^$/I=<($G$-$J$$LdBj$X$NBP:v(B
+			 ;; (NTEmacs 22.1, 23.1)
+			 (encode-coding-string (cdr cons) 'shift_jis)
+		       (cdr cons))))
        ((and (vectorp (car-safe list))
 	     (setq cons (assoc (aref (car list) 0) skk-e21-menu-resource-ja)))
-	(aset (car list) 0 (skk-e21-encode-string (cdr cons)))))
+	(aset (car list) 0 (if (and running-ntemacs
+				    (member (aref (car list) 0)
+					    '("Hiragana" "Katakana"
+					      "Hankaku alphabet" "Zenkaku alphabet")))
+			       ;; NTEmacs $B$G(B Widget $BIU$-%a%K%e!<%"%$%F%`$N(B
+			       ;; $BF|K\8l$,$&$^$/I=<($G$-$J$$LdBj$X$NBP:v(B
+			       ;; (NTEmacs 22.1, 23.1)
+			       (encode-coding-string (cdr cons) 'shift_jis)
+			     (cdr cons)))))
       (setq list (cdr list)))))
 
 (defun skk-e21-mouse-position ()
@@ -407,7 +420,11 @@
 			  ;; (cdr avoid-destination)
 			  0))
     ;;
-    (unless (eq skk-tooltip-mouse-behavior 'follow)
+    (cond
+     ((eq skk-tooltip-mouse-behavior 'follow)
+      (setq tooltip-info (skk-tooltip-resize-text text)
+	    text (car tooltip-info)))
+     (t
       ;; $B%^%&%9%]%$%s%?$K0MB8$;$:(B tooptip $B$N0LCV$r7hDj$9$k!#(B
       (setq edges (window-inside-pixel-edges
 		   (if skk-isearch-switch
@@ -495,7 +512,7 @@
 			  (* (frame-pixel-width)))))
 	  (when (and (<= left mouse-x) (<= mouse-x right))
 	    ;; $B%^%&%9%]%$%s%?$HHo$j$=$&$J$H$-(B
-	    (setq left (- left (- right mouse-x) fontsize))))))
+	    (setq left (- left (- right mouse-x) fontsize)))))))
     ;;
     (setq parameters (if (eq skk-tooltip-mouse-behavior 'follow)
 			 skk-tooltip-parameters
@@ -556,10 +573,8 @@
 	    (setq params (tooltip-set-param params 'border-color fg)))
 	  (when (stringp bg)
 	    (setq params (tooltip-set-param params 'background-color bg))))
-	(unless (ignore-errors
-		  (or (get-text-property 0 'face text)
-		      (get-text-property 2 'face text)))
-	  (setq text (propertize text 'face 'tooltip)))
+	(when (facep skk-tooltip-face)
+	  (setq text (propertize text 'face skk-tooltip-face)))
 	;; $B%_%K%P%C%U%!$K$$$k$H$-M>7W$J%a%C%;!<%8$r%/%j%"$9$k(B
 	(when (or skk-isearch-switch
 		  (skk-in-minibuffer-p))
