@@ -88,6 +88,7 @@
 
 (defun skk-hint-setup-hint ()
   (cond ((eq skk-hint-state 'kana)
+	 (skk-kana-cleanup t)
 	 (let ((hint (buffer-substring-no-properties
 		      skk-hint-start-point (point))))
 	   (unless (string= hint "")
@@ -114,7 +115,7 @@
 
 (defadvice skk-insert (around skk-hint-ad activate)
   (cond ((and skk-henkan-mode
-	      (eq last-command-char skk-hint-start-char)
+	      (eq (skk-last-command-char) skk-hint-start-char)
 	      (not skk-hint-state))
 	 (skk-with-point-move
 	  (when (featurep 'skk-dcomp)
@@ -124,7 +125,7 @@
 	  (setq skk-hint-state 'kana
 		skk-hint-inhibit-kakutei t)))
 	((and (eq skk-hint-state 'kana)
-	      (eq last-command-char skk-start-henkan-char))
+	      (eq (skk-last-command-char) skk-start-henkan-char))
 	 (skk-with-point-move
 	  (skk-hint-setup-hint)
 	  (delete-region skk-hint-start-point (point))
@@ -133,12 +134,13 @@
 	  (setq skk-henkan-list nil)
 	  (skk-start-henkan arg)))
 	((and (eq skk-hint-state 'kana)
-	      (memq last-command-char skk-set-henkan-point-key))
+	      (memq (skk-last-command-char) skk-set-henkan-point-key))
 	 (skk-with-point-move
 	  (setq skk-hint-end-point (point))
 	  (setq skk-hint-state 'okuri)
-	  (setq last-command-char (skk-downcase last-command-char))
-	  (setq skk-hint-okuri-char (char-to-string last-command-char))
+	  (skk-set-last-command-char (skk-downcase (skk-last-command-char)))
+	  (setq skk-hint-okuri-char (skk-char-to-unibyte-string
+				     (skk-last-command-char)))
 	  (skk-kana-input arg)
 	  (when (skk-jisx0208-p (char-before))
 	    (skk-hint-setup-hint)
