@@ -5,9 +5,9 @@
 
 ;; Author: Enami Tsugutomo <enami@ba2.so-net.or.jp>
 ;; Maintainer: SKK Development Team <skk@ring.gr.jp>
-;; Version: $Id: skk-isearch.el,v 1.56 2007/08/02 08:22:35 skk-cvs Exp $
+;; Version: $Id: skk-isearch.el,v 1.60 2009/11/02 03:56:13 skk-cvs Exp $
 ;; Keywords: japanese, mule, input method
-;; Last Modified: $Date: 2007/08/02 08:22:35 $
+;; Last Modified: $Date: 2009/11/02 03:56:13 $
 
 ;; This file is part of Daredevil SKK.
 
@@ -135,7 +135,7 @@
   "Show isearch message."
   (skk-isearch-incomplete-message
    (if (string= skk-prefix "")
-       (char-to-string last-command-char)
+       (skk-char-to-unibyte-string (skk-last-command-char))
      skk-prefix)))
 
 (defun skk-isearch-current-mode ()
@@ -249,7 +249,7 @@ kakutei'ed and erase the buffer contents."
   (set skk-isearch-overriding-local-map skk-isearch-mode-map)
   ;; Input Method $B$H$7$F(B SKK $B$r;H$C$F$$$k>l9g$NBP:v(B
   (static-when
-      (memq skk-emacs-type '(mule4 mule5))
+      (memq skk-emacs-type '(mule4 mule5 mule6))
     (when (and current-input-method
 	       (string-match "^japanese-skk" current-input-method))
       (let* ((method current-input-method)
@@ -302,7 +302,7 @@ kakutei'ed and erase the buffer contents."
        (skk-jisx0208-latin-mode-on))))
   ;; Input Method $B$H$7$F(B SKK $B$r;H$C$F$$$k>l9g$NBP:v(B
   (static-when
-      (memq skk-emacs-type '(mule4 mule5))
+      (memq skk-emacs-type '(mule4 mule5 mule6))
     (when (string-match "^japanese-skk" (format "%s" default-input-method))
       (with-current-buffer (get-buffer-create skk-isearch-working-buffer)
 	(inactivate-input-method))))
@@ -346,14 +346,26 @@ Optional argument PREFIX is appended if given."
   ;; printable chars.
   (do ((c ?\040 (1+ c)))
       ((>= c ?\177))
-    (define-key map (char-to-string c) 'skk-isearch-wrapper))
+    (define-key map (skk-char-to-unibyte-string c) 'skk-isearch-wrapper))
+
+  (when skk-j-mode-function-key-usage
+    (define-key map [f1] 'skk-isearch-wrapper)
+    (define-key map [f2] 'skk-isearch-wrapper)
+    (define-key map [f3] 'skk-isearch-wrapper)
+    (define-key map [f4] 'skk-isearch-wrapper)
+    (define-key map [f5] 'skk-isearch-wrapper)
+    (define-key map [f6] 'skk-isearch-wrapper)
+    (define-key map [f7] 'skk-isearch-wrapper)
+    (define-key map [f8] 'skk-isearch-wrapper)
+    (define-key map [f9] 'skk-isearch-wrapper)
+    (define-key map [f10] 'skk-isearch-wrapper))
 
   ;; control chars for skk.
   (define-key map "\C-g" 'skk-isearch-keyboard-quit)
   (define-key map skk-kakutei-key 'skk-isearch-newline)
   (define-key map "\C-m" 'skk-isearch-exit)
   (when (characterp skk-previous-candidate-char)
-    (define-key map (char-to-string skk-previous-candidate-char)
+    (define-key map (skk-char-to-unibyte-string skk-previous-candidate-char)
       'skk-isearch-wrapper))
 
   ;; C-x map for skk.
@@ -678,14 +690,14 @@ If the current mode is different from previous, remove it first."
 $B$,;2>H$5$l$k!#(B"
   (interactive)
   (let ((digit (or digit
-		   (- (logand last-command-char ?\177) ?0)))
+		   (- (logand (skk-last-command-char) ?\177) ?0)))
 	(event (next-command-event nil (skk-isearch-incomplete-message))))
     (cond
      ((equal event (character-to-event ?\ ))
       ;; XEmacs $B$G$O(B eq $B$K$O$J$i$J$$(B
       (with-current-buffer (get-buffer-create skk-isearch-working-buffer)
 	(when (eq skk-henkan-mode 'on)
-	  (let ((last-command-char skk-start-henkan-char))
+	  (skk-bind-last-command-char skk-start-henkan-char
 	    (skk-start-henkan 1 digit))))
       (skk-isearch-mode-message)
       (skk-isearch-wrapper-1))
