@@ -1,12 +1,13 @@
 ;;; skk-macs.el --- macros and inline functions commonly used in SKK -*- coding: iso-2022-jp -*-
 
-;; Copyright (C) 1999-2007  SKK Development Team <skk@ring.gr.jp>
+;; Copyright (C) 1999-2008  SKK Development Team <skk@ring.gr.jp>
+;; Copyright (C) 1993, 2000 Free Software Foundation, Inc.
 
 ;; Author: SKK Development Team <skk@ring.gr.jp>
 ;; Maintainer: SKK Development Team <skk@ring.gr.jp>
-;; Version: $Id: skk-macs.el,v 1.120 2007/08/13 23:37:24 skk-cvs Exp $
+;; Version: $Id: skk-macs.el,v 1.136 2009/11/02 03:56:13 skk-cvs Exp $
 ;; Keywords: japanese, mule, input method
-;; Last Modified: $Date: 2007/08/13 23:37:24 $
+;; Last Modified: $Date: 2009/11/02 03:56:13 $
 
 ;; This file is part of Daredevil SKK.
 
@@ -54,6 +55,8 @@ Otherwise, return result of last FORM."
 ;;;###autoload
 (put 'dolist 'lisp-indent-function 1)
 (put 'dolist 'defmacro-maybe (eq skk-emacs-type 'mule4))
+
+;; From GNU Emacs 21.
 (defmacro-maybe dolist (spec &rest body)
   "(dolist (VAR LIST [RESULT]) BODY...): loop over a list.
 Evaluate BODY with VAR bound to each car from LIST, in turn.
@@ -72,6 +75,8 @@ Then evaluate RESULT to get return value, default nil."
 ;;;###autoload
 (put 'dotimes 'lisp-indent-function 1)
 (put 'dotimes 'defmacro-maybe (eq skk-emacs-type 'mule4))
+
+;; From GNU Emacs 21.
 (defmacro-maybe dotimes (spec &rest body)
   "(dotimes (VAR COUNT [RESULT]) BODY...): loop a certain number of times.
 Evaluate BODY with VAR bound to successive integers running from 0,
@@ -191,7 +196,7 @@ ENGLISH $B$r%(%3!<%(%j%"$KI=<($7!"%(%i!<$rH/@8$5$;$k!#(B ARG $B$O(B `error'
 				   japanese english)))
 
 (defmacro skk-y-or-n-p (japanese english)
-  "$B%f!<%6$K(B \"y or n\" $B$N<ALd$r$7!"Ez$($,(B \y\" $B$@$C$?$i(B t $B$rJV$9!#(B
+  "$B%f!<%6$K(B \"y or n\" $B$N<ALd$r$7!"Ez$($,(B \"y\" $B$@$C$?$i(B t $B$rJV$9!#(B
 `skk-japanese-message-and-error' $B$,(B non-nil $B$G$"$l$P(B JAPANESE $B$r(B nil $B$G$"$l(B
 $B$P(B ENGLISH $B$r%W%m%s%W%H$H$7$F(B `y-or-n-p' $B$r<B9T$9$k!#(B"
   (list 'y-or-n-p (list 'if 'skk-japanese-message-and-error
@@ -260,6 +265,8 @@ MARKER $B$,(B nil $B$@$C$?$i!"?75,%^!<%+!<$r:n$C$FBeF~$9$k!#(B"
 
 ;;; functions.
 ;; version dependent
+
+;; From XEmacs 21.5.
 (defun-maybe propertize (string &rest properties)
   "Return a copy of STRING with text properties added.
 First argument is the string to copy.
@@ -271,6 +278,19 @@ properties to add to the result."
 			 str)
     str))
 
+(defun-maybe substring-no-properties (string &optional from to)
+  "Return a substring of string, without text properties.
+It starts at index from and ending before to.
+to may be nil or omitted; then the substring runs to the end of string.
+If from is nil or omitted, the substring starts at the beginning of string.
+If from or to is negative, it counts from the end.
+
+With one argument, just copy string without its properties."
+  (let ((substr (copy-sequence (substring string (or from 0) to))))
+    (set-text-properties 0 (length substr) nil substr)
+    substr))
+
+;; From GNU Emacs 22.1.
 (defun-maybe replace-regexp-in-string (regexp rep string &optional
 					      fixedcase literal subexp start)
   "Replace all matches for REGEXP with REP in STRING.
@@ -329,21 +349,10 @@ and replace a sub-expression, e.g.
       (setq matches (cons (substring string start l) matches)) ; leftover
       (apply #'concat (nreverse matches)))))
 
-(defun-maybe substring-no-properties (string &optional from to)
-  "Return a substring of string, without text properties.
-It starts at index from and ending before to.
-to may be nil or omitted; then the substring runs to the end of string.
-If from is nil or omitted, the substring starts at the beginning of string.
-If from or to is negative, it counts from the end.
-
-With one argument, just copy string without its properties."
-  (let ((substr (copy-sequence (substring string (or from 0) to))))
-    (set-text-properties 0 (length substr) nil substr)
-    substr))
-
 (static-unless (featurep 'xemacs)
   (fmakunbound 'next-command-event))
 
+;; For GNU Emacs.
 (defun-maybe next-command-event (&optional event prompt)
   "Read an event object from the input stream.
 If EVENT is non-nil, it should be an event object and will be filled
@@ -353,6 +362,9 @@ If PROMPT is non-nil, it should be a string and will be displayed in
 the echo area while this function is waiting for an event."
   (read-event prompt))
 
+;; For GNU Emacs 20.7.
+(defalias-maybe 'plist-member 'widget-plist-member)
+
 (defsubst skk-sit-for (seconds &optional nodisplay)
   "`sit-for' $B$N(B Emacsen $B$K$h$k0c$$$r5[<}$9$k!#(B"
   (static-cond
@@ -379,29 +391,67 @@ the echo area while this function is waiting for an event."
     ;; FSF Emacs on X Window System.
     (and window-system (x-display-color-p)))))
 
-(defsubst skk-char-to-string (char)
+(defsubst skk-char-to-unibyte-string (char)
   (ignore-errors
-    (char-to-string char)))
+    (static-cond
+     ((memq skk-emacs-type '(mule6))
+      (string-make-unibyte (char-to-string char)))
+     (t
+      (char-to-string char)))))
 
 (defsubst skk-ascii-char-p (char)
-  (eq (char-charset char) 'ascii))
+  (static-cond
+   ((memq skk-emacs-type '(mule6))
+    (eq (char-charset char skk-charset-list) 'ascii))
+   (t
+    (eq (char-charset char) 'ascii))))
 
 (defsubst skk-jisx0208-p (char)
-  (eq (char-charset char) 'japanese-jisx0208))
+  (static-cond
+   ((memq skk-emacs-type '(mule6))
+    (eq (char-charset char skk-charset-list) 'japanese-jisx0208))
+   (t
+    (eq (char-charset char) 'japanese-jisx0208))))
 
 (defsubst skk-jisx0213-p (char)
-  (and (featurep 'jisx0213)
-       (memq (char-charset char)
-	     '(japanese-jisx0213-1 japanese-jisx0213-2))))
+  (static-cond
+   ((memq skk-emacs-type '(mule6))
+       (memq (char-charset char skk-charset-list)
+	     '(japanese-jisx0213.2004-1
+	       japanese-jisx0213-1
+	       japanese-jisx0213-2)))
+   (t
+    (and (featurep 'jisx0213)
+	 (memq (char-charset char)
+	       '(japanese-jisx0213-1 japanese-jisx0213-2))))))
 
 (defsubst skk-char-octet (ch &optional n)
+  (or (nth (if n
+	       (1+ n)
+	     1)
+	   (skk-split-char ch))
+      0))
+
+(defun skk-split-char (ch)
   (static-cond
-   ((eq skk-emacs-type 'xemacs)
-    (or (nth (if n (1+ n) 1) (split-char ch))
-	0))
+   ((memq skk-emacs-type '(mule6))
+    ;; C $B$N(B split-char $B$HF1MM$N5!G=$@$,!"(Bchar-charset $B$N8F=P$7$K$*$$$F(B
+    ;; $BJ8;z=89g$NA*Br;h$r(B skk-charset-list $B$K4^$^$l$k$b$N$K@)8B$9$k!#(B
+    ;; $B$3$l$ONc$($P!"(Bjapanese-jisx0208 $B$NJ8;z$,(B unicode-bmp $B$KB0$9$k!"(B
+    ;; $B$HH=Dj$5$l$k$h$&$J>u67$r2sHr$9$k!#(B
+    (let* ((charset (char-charset ch skk-charset-list))
+	   (code (encode-char ch charset))
+	   (dimension (charset-dimension charset))
+	   val)
+      (while (> dimension 0)
+	(setq val (cons (logand code 255) ;; 0xFF
+			val))
+	(setq code (lsh code -8))
+	(setq dimension (1- dimension)))
+      (cons charset val)))
    (t
-    ;; FSF Emacs.
-    (char-octet ch n))))
+    ;; Emacs 22 $B0JA0!"$*$h$S(B XEmacs
+    (split-char ch))))
 
 ;; this one is called once in skk-kcode.el, too.
 (defsubst skk-charsetp (object)
@@ -419,7 +469,7 @@ the echo area while this function is waiting for an event."
     (if (stringp indicator)
 	indicator
       (cdr indicator)))
-   ((eq skk-emacs-type 'mule5)
+   ((memq skk-emacs-type '(mule5 mule6))
     (if no-properties
 	(with-temp-buffer
 	  (insert indicator)
@@ -434,7 +484,7 @@ the echo area while this function is waiting for an event."
    ((eq skk-emacs-type 'xemacs)
     (cons (cdr (assq mode skk-xemacs-extent-alist))
 	  string))
-   ((memq skk-emacs-type '(mule5))
+   ((memq skk-emacs-type '(mule5 mule6))
     (if (and window-system
 	     (not (eq mode 'default)))
 	(apply 'propertize string
@@ -456,7 +506,7 @@ BUFFER defaults to the current buffer."
   (static-cond
    ((eq skk-emacs-type 'xemacs)
     (face-proportional-p face))
-   ((eq skk-emacs-type 'mule5)
+   ((memq skk-emacs-type '(mule5 mule6))
     (or (face-equal face 'variable-pitch)
 	(eq (face-attribute face :inherit) 'variable-pitch)))))
 
@@ -480,6 +530,36 @@ BUFFER defaults to the current buffer."
 	(setq keys (recent-keys))
 	(vector (aref keys (1- (length keys)))))))))
 
+;; Emacs 23 or later shows warning for the use of last-command-char,
+;; which is an obsolete variable alias for last-command-event.
+;; What is annoying is incompatibility between FSF Emacs and XEmacs.
+;; In XEmacs, last-command-event is an event, not a character, whereas
+;; last-command-char is a character equivalent to last-command-event.
+(defsubst skk-last-command-char ()
+  (static-cond
+   ((eq skk-emacs-type 'xemacs)
+    last-command-char)
+   (t
+    last-command-event)))
+
+(defsubst skk-set-last-command-char (char)
+  (let ((variable (static-cond
+		   ((eq skk-emacs-type 'xemacs)
+		    'last-command-char)
+		   (t
+		    'last-command-event))))
+    (set variable char)))
+
+(put 'skk-bind-last-command-char 'lisp-indent-function 1)
+(defmacro skk-bind-last-command-char (char &rest body)
+  (let ((variable (cond ((eq skk-emacs-type 'xemacs)
+			 'last-command-char)
+			(t
+			 'last-command-event))))
+    `(let ((,variable ,char))
+       (progn
+	 ,@body))))
+
 ;;; version independent
 (defsubst skk-cursor-set (&optional color force)
   (when (or skk-use-color-cursor
@@ -703,15 +783,44 @@ BUFFER defaults to the current buffer."
   "$B%+%l%s%H%P%C%U%!$,%_%K%P%C%U%!$+$I$&$+$r%A%'%C%/$9$k!#(B"
   (eq (current-buffer) (window-buffer (minibuffer-window))))
 
+(defun skk-window-body-height ()
+  (static-if (fboundp 'window-body-height) ; emacs21 $B$K$O$J$$(B
+      (window-body-height)
+    (- (window-height)
+       (if mode-line-format 1 0)
+       (static-cond ((not (boundp 'header-line-format)) 0) ; emacs20 $B$K$O$J$$(B
+		    (t (if header-line-format 1 0))))))
+
+(defun skk-screen-column ()
+  (- (current-column)
+     (save-excursion
+       (vertical-motion 0)
+       (current-column))))
+
+(defun skk-move-to-screen-column (col)
+  (move-to-column (+ (current-column)
+		     (- col (skk-screen-column))))
+  (skk-screen-column))
+
 (defun skk-insert-prefix (&optional char)
   "`skk-echo' $B$,(B non-nil $B$G$"$l$P%+%l%s%H%P%C%U%!$K(B `skk-prefix' $B$rA^F~$9$k!#(B"
   (when skk-echo
     ;; skk-prefix $B$O%"%s%I%%$NBP>]$H$J$i$J$$$N$G(B
     ;; $B%P%C%U%!$KA^F~$5$l$kI,MW$,$J$$!#(B
     (setq skk-prefix-overlay (make-overlay (point) (point)))
-    (overlay-put skk-prefix-overlay 'after-string (or char skk-prefix))
-    (static-when (eq skk-emacs-type 'mule4)
-      (redraw-frame (selected-frame)))))
+    (let ((prefix (or char skk-prefix)))
+      (when (and skk-use-face (not skk-henkan-mode))
+	(setq prefix
+	      (propertize prefix 'face
+			  (cond ((and skk-j-mode (not skk-katakana))
+				 'skk-prefix-hiragana-face)
+				(skk-katakana
+				 'skk-prefix-katakana-face)
+				(skk-jisx0201-mode
+				 'skk-prefix-jisx0201-face)))))
+      (overlay-put skk-prefix-overlay 'after-string prefix)
+      (static-when (eq skk-emacs-type 'mule4)
+	(redraw-frame (selected-frame))))))
 
 (defsubst skk-string-lessp-in-coding-system (str1 str2 coding-system)
   (string< (encode-coding-string str1 coding-system)
