@@ -7,9 +7,9 @@
 
 ;; Author: Masahiko Sato <masahiko@kuis.kyoto-u.ac.jp>
 ;; Maintainer: SKK Development Team <skk@ring.gr.jp>
-;; Version: $Id: skk-tut.el,v 1.65 2007/04/22 02:38:26 skk-cvs Exp $
+;; Version: $Id: skk-tut.el,v 1.68 2010/01/30 03:44:19 skk-cvs Exp $
 ;; Keywords: japanese, mule, input method
-;; Last Modified: $Date: 2007/04/22 02:38:26 $
+;; Last Modified: $Date: 2010/01/30 03:44:19 $
 
 ;; This file is part of Daredevil SKK.
 
@@ -352,7 +352,7 @@
   (let ((map (make-sparse-keymap))
 	(c 32))
     (while (< c 127)
-      (define-key map (char-to-string c) 'skk-insert)
+      (define-key map (skk-char-to-unibyte-string c) 'skk-insert)
       (setq c (1+ c)))
     (setq skktut-j-mode-map map)))
 
@@ -364,7 +364,7 @@
 	(i 0))
     (while (< i 128)
       (if (aref skk-jisx0208-latin-vector i)
-	  (define-key map (char-to-string i) 'skk-jisx0208-latin-insert))
+	  (define-key map (skk-char-to-unibyte-string i) 'skk-jisx0208-latin-insert))
 	(setq i (1+ i)))
       (define-key map "\C-j" 'skk-kakutei)
       (setq skktut-jisx0208-latin-mode-map map)))
@@ -413,7 +413,7 @@
 
 (defadvice skk-insert (before skktut-ad disable)
   "SKK $B%A%e!<%H%j%"%kMQ%"%I%P%$%9IU!#(B"
-  (when (and (memq last-command-char skk-set-henkan-point-key)
+  (when (and (memq (skk-last-command-char) skk-set-henkan-point-key)
 	     (> 12 skktut-question-count))
     (skktut-error "$B$+$J(B/$B%+%J%b!<%I$G$O!"1QBgJ8;z$O$^$@;H$($^$;$s(B"
 		  "Cannot use upper case character in kana/katakana mode")))
@@ -846,9 +846,8 @@ tutorial /$B%A%e!<%H%j%"%k(B/
 
 (defun skktut-next-answer-buffer ()
   (save-match-data
-    (save-excursion
+    (with-current-buffer skktut-question-buffer
       ;; first get right answer in `skktut-question-buffer'.
-      (set-buffer skktut-question-buffer)
       (goto-char (point-max))
       (search-backward "\n>>")
       (forward-char 1)
