@@ -4,9 +4,9 @@
 
 ;; Author: NAKAJIMA Mikio <minakaji@namazu.org>
 ;; Maintainer: SKK Development Team <skk@ring.gr.jp>
-;; Version: $Id: skk-version.el,v 1.30 2007/04/22 02:38:27 skk-cvs Exp $
+;; Version: $Id: skk-version.el,v 1.34 2010/01/27 06:26:42 skk-cvs Exp $
 ;; Keywords: japanese, mule, input method
-;; Last Modified: $Date: 2007/04/22 02:38:27 $
+;; Last Modified: $Date: 2010/01/27 06:26:42 $
 
 ;; This file is part of Daredevil SKK.
 
@@ -35,7 +35,7 @@
 (provide 'skk-version) ; have to declare in the top.
 
 (product-provide 'skk-version
-  (product-define "Daredevil SKK" nil '(13 0 91) "Hattori"))
+  (product-define "Daredevil SKK" nil '(14 0 91) "Syounai"))
 
 ;; set version-string
 (if (fboundp 'product-version-as-string)
