@@ -7,9 +7,9 @@
 ;; Author: Masahiko Sato <masahiko@kuis.kyoto-u.ac.jp>,
 ;;         Murata Shuuichirou <mrt@notwork.org>
 ;; Maintainer: SKK Development Team <skk@ring.gr.jp>
-;; Version: $Id: skk-viper.el,v 1.36 2007/06/28 13:10:59 skk-cvs Exp $
+;; Version: $Id: skk-viper.el,v 1.42 2009/07/11 02:15:39 skk-cvs Exp $
 ;; Keywords: japanese, mule, input method
-;; Last Modified: $Date: 2007/06/28 13:10:59 $
+;; Last Modified: $Date: 2009/07/11 02:15:39 $
 
 ;; This file is part of Daredevil SKK.
 
@@ -43,14 +43,14 @@
 
 ;;; macros and inline functions.
 (defmacro skk-viper-advice-select (viper vip arg body)
-  (` (if skk-viper-use-vip-prefix
-	 (defadvice (, vip) (, arg) (,@ body))
-       (defadvice (, viper) (, arg) (,@ body)))))
+  `(if skk-viper-use-vip-prefix
+	 (defadvice ,vip ,arg ,@body)
+       (defadvice ,viper ,arg ,@body)))
 
 (setq skk-kana-cleanup-command-list
       (cons
        (if skk-viper-use-vip-prefix
-	   'vip-del-backward-char-in-insert
+	   'vip-delete-backward-char
 	 'viper-del-backward-char-in-insert)
        skk-kana-cleanup-command-list))
 
@@ -67,19 +67,26 @@
 ;; `viper-insert-state-cursor-color'?
 (when (boundp 'viper-insert-state-cursor-color)
   (defadvice skk-cursor-current-color (around skk-viper-cursor-ad activate)
-    "vi-state $B$N$H$-$O!"(BSKK $B%b!<%I$K$J$C$F$$$F$b%G%#%U%)%k%H%+!<%=%k$rJV$9!#(B"
+    "vi-state $B0J30$G3n$D(B SKK $B%b!<%I$N$H$-$N$_(B SKK $BM3Mh$N%+!<%=%k?'$rJV$9!#(B"
     (cond
      ((not skk-use-color-cursor)
       ad-do-it)
      ((or (and (boundp 'viper-current-state)
 	       (eq viper-current-state 'vi-state))
-	  (and (boundp 'vip-current-state)
-	       (eq vip-current-state 'vi-state)))
+	  (and (boundp 'vip-current-mode)
+	       (eq vip-current-mode 'vi-mode)))
       (setq ad-return-value skk-cursor-default-color))
      ((not skk-mode)
       (setq viper-insert-state-cursor-color
 	    skk-viper-saved-cursor-color)
-      ad-do-it)
+      (setq ad-return-value
+	    (cond
+	     ((eq viper-current-state 'insert-state)
+	      viper-insert-state-cursor-color)
+	     ((eq viper-current-state 'replace-state)
+	      viper-replace-overlay-cursor-color)
+	     ((eq viper-current-state 'emacs-state)
+	      viper-emacs-state-cursor-color))))
      (t
       ad-do-it
       (setq viper-insert-state-cursor-color ad-return-value))))
@@ -90,22 +97,20 @@
 	     '(vip-Append
 	       vip-Insert
 	       vip-insert
-	       vip-intercept-ESC-key
+	       vip-escape-to-emacs
 	       vip-open-line)
-	   '(viper-Append
-	     viper-Insert
-	     viper-hide-replace-overlay
-	     viper-insert
-	     viper-intercept-ESC-key
-	     viper-open-line))))
+	   '(viper-change-state-to-insert
+	     viper-change-state-to-replace
+	     viper-change-state-to-vi
+	     viper-change-state-to-emacs
+	     viper-insert-state-post-command-sentinel))))
     (dolist (func funcs)
       (eval
-       (`
-	(defadvice (, (intern (symbol-name func)))
+       `(defadvice ,(intern (symbol-name func))
 	  (after skk-viper-cursor-ad activate)
 	  "Set cursor color which represents skk mode."
 	  (when skk-use-color-cursor
-	    (skk-cursor-set)))))))
+	    (skk-cursor-set))))))
 
   (let ((funcs '(skk-abbrev-mode
 		 skk-jisx0208-latin-mode
@@ -113,23 +118,18 @@
 		 skk-toggle-kana)))
     (dolist (func funcs)
       (eval
-       (`
-	(defadvice (, (intern (symbol-name func)))
+       `(defadvice ,(intern (symbol-name func))
 	  (after skk-viper-cursor-ad activate)
 	  "\
-viper-insert-state-cursor-color $B$r(B SKK $B$NF~NO%b!<%I$N%+!<%=%k?'$H9g$o$;$k!#(B"
+`viper-insert-state-cursor-color' $B$r(B SKK $B$NF~NO%b!<%I$N%+!<%=%k?'$H9g$o$;$k!#(B"
 	  (when skk-use-color-cursor
 	    (setq viper-insert-state-cursor-color
-		  (skk-cursor-current-color))))))))
+		  (skk-cursor-current-color)))))))
 
   (defadvice skk-mode (after skk-viper-cursor-ad activate)
-    "\
-viper-insert-state-cursor-color $B$r(B SKK $B$NF~NO%b!<%I$N%+!<%=%k?'$H9g$o$;$k!#(B"
+    "Set cursor color which represents skk mode."
     (when skk-use-color-cursor
-      (setq viper-insert-state-cursor-color
-	    (if skk-mode
-		(skk-cursor-current-color)
-	      skk-viper-saved-cursor-color))))
+      (skk-cursor-set)))
 
   (defadvice skk-kakutei (after skk-viper-cursor-ad activate)
     (setq viper-insert-state-cursor-color skk-cursor-hiragana-color)))
@@ -144,9 +144,9 @@ viper-insert-state-cursor-color $B$r(B SKK $B$NF~NO%b!<%I$N%+!<%=%k?'$H9g$o$;
 		  'append))))
    (t
     (skk-defadvice read-from-minibuffer (before skk-viper-ad activate)
-      "minibuffer-setup-hook $B$K(B update-buffer-local-frame-params $B$r%U%C%/$9$k!#(B
-viper-read-string-with-history $B$O(B minibuffer-setup-hook $B$r4X?t%m!<%+%k(B
-$B$K$7$F$7$^$&$N$G!"M=$a(B minibuffer-setup-hook $B$K$+$1$F$*$$$?%U%C%/$,L58z(B
+      "`minibuffer-setup-hook' $B$K(B `update-buffer-local-frame-params' $B$r%U%C%/$9$k!#(B
+`viper-read-string-with-history' $B$O(B `minibuffer-setup-hook' $B$r4X?t%m!<%+%k(B
+$B$K$7$F$7$^$&$N$G!"M=$a(B `minibuffer-setup-hook' $B$K$+$1$F$*$$$?%U%C%/$,L58z(B
 $B$H$J$k!#(B"
       (when skk-use-color-cursor
 	;; non-command subr.
@@ -163,10 +163,10 @@ viper-read-string-with-history $B$O(B minibuffer-setup-hook $B$r4X?t%m!<%+%k
 				    'skk-add-skk-pre-command))
 
 (skk-viper-advice-select
- viper-forward-word-kernel vip-forward-word-kernel
+ viper-forward-word-kernel vip-forward-word
  (around skk-ad activate)
  ("SKK $B%b!<%I$,%*%s$G!"%]%$%s%H$ND>8e$NJ8;z$,(B JISX0208/JISX0213 $B$@$C$?$i(B\
- forward-word $B$9$k!#(B"
+ `forward-word' $B$9$k!#(B"
   (if (and skk-mode
 	   (or (skk-jisx0208-p (following-char))
 	       (skk-jisx0213-p (following-char))))
@@ -174,76 +174,84 @@ viper-read-string-with-history $B$O(B minibuffer-setup-hook $B$r4X?t%m!<%+%k
     ad-do-it)))
 
 (skk-viper-advice-select
- viper-backward-word-kernel vip-backward-word-kernel
+ viper-backward-word-kernel vip-backward-word
  (around skk-ad activate)
  ("SKK $B%b!<%I$,%*%s$G!"%]%$%s%H$ND>A0$NJ8;z$,(B JISX0208/JISX0213 $B$@$C$?$i(B\
- backward-word $B$9$k!#(B"
+ `backward-word' $B$9$k!#(B"
   (if (and skk-mode (or (skk-jisx0208-p (preceding-char))
 			(skk-jisx0213-p (preceding-char))))
       (backward-word (ad-get-arg 0))
     ad-do-it)))
 
-;; please sync with advice to delete-backward-char
+;; please sync with `skk-delete-backward-char'
 (skk-viper-advice-select
- viper-del-backward-char-in-insert vip-del-backward-char-in-insert
+ viper-del-backward-char-in-insert vip-delete-backward-char
  (around skk-ad activate)
- ("$B"'%b!<%I$G(B skk-delete-implies-kakutei $B$,(B non-nil $B$@$C$?$iD>A0$NJ8;z$r>C$7$F(B\
-$B3NDj$9$k!#(B
-$B"'%b!<%I$G(B skk-delete-implies-kakutei $B$,(B nil $B$@$C$?$iA08uJd$rI=<($9$k!#(B
-$B"&%b!<%I$@$C$?$i3NDj$9$k!#(B
+ ("$B"'%b!<%I$G(B `skk-delete-implies-kakutei' $B$J$iD>A0$NJ8;z$r>C$7$F3NDj$9$k!#(B
+$B"'%b!<%I$G(B `skk-delete-implies-kakutei' $B$,(B nil $B$@$C$?$iA08uJd$rI=<($9$k!#(B
+$B"&%b!<%I$G(B`$B"&(B'$B$h$j$bA0$N%]%$%s%H$G<B9T$9$k$H3NDj$9$k!#(B
 $B3NDjF~NO%b!<%I$G!"$+$J%W%l%U%#%C%/%9$NF~NOCf$J$i$P!"$+$J%W%l%U%#%C%/%9$r>C$9!#(B"
-  (let ((count (or (prefix-numeric-value (ad-get-arg 0)) 1)))
-    (cond
-     ((eq skk-henkan-mode 'active)
-      (if (and (not skk-delete-implies-kakutei)
-	       (= (+ skk-henkan-end-point (length skk-henkan-okurigana))
-		  (point)))
-	  (skk-previous-candidate)
-	;;(if skk-use-face (skk-henkan-face-off))
-	;; overwrite-mode $B$G!"%]%$%s%H$,A43QJ8;z$K0O$^$l$F$$$k$H(B
-	;; $B$-$K(B delete-backward-char $B$r;H$&$H!"A43QJ8;z$O>C$9$,H>(B
-	;; $B3QJ8;zJ,$7$+(B backward $BJ}8~$K%]%$%s%H$,La$i$J$$(B (Emacs
-	;; 19.31 $B$K$F3NG'(B)$B!#JQ49Cf$N8uJd$KBP$7$F$O(B
-	;; delete-backward-char $B$GI,$:A43QJ8;z(B 1 $BJ8;zJ,(B backward
-	;; $BJ}8~$KLa$C$?J}$,NI$$!#(B
-	(if overwrite-mode
-	    (progn
-	      (backward-char count)
-	      (delete-char count))
-	  ad-do-it)
-	;; XXX assume skk-prefix has no multibyte chars.
-	(if (> (length skk-prefix) count)
-	    (setq skk-prefix (substring skk-prefix
-					0 (- (length skk-prefix) count)))
-	  (setq skk-prefix ""))
-	(when (>= skk-henkan-end-point (point))
-	  (if (eq skk-delete-implies-kakutei 'dont-update)
-	      (let ((skk-update-jisyo-function #'ignore))
-		(skk-kakutei))
-	    (skk-kakutei)))))
-     ((and (eq skk-henkan-mode 'on)
-	   (>= skk-henkan-start-point (point)))
-      (setq skk-henkan-count 0)
-      (skk-kakutei))
-     ;; $BF~NOCf$N8+=P$78l$KBP$7$F$O(B delete-backward-char $B$GI,$:A43QJ8;z(B 1
-     ;; $BJ8;zJ,(B backward $BJ}8~$KLa$C$?J}$,NI$$!#(B
-     ((and (eq skk-henkan-mode 'on)
-	   overwrite-mode)
-      (backward-char count)
-      (delete-char count))
-     (t
-      (if (string= skk-prefix "")
-	  ad-do-it
-	(skk-erase-prefix 'clean)))))))
+  (skk-with-point-move
+   (let ((count (or (prefix-numeric-value (ad-get-arg 0)) 1)))
+     (cond
+      ((eq skk-henkan-mode 'active)
+       (if (and (not skk-delete-implies-kakutei)
+		(= (+ skk-henkan-end-point (length skk-henkan-okurigana))
+		   (point)))
+	   (skk-previous-candidate)
+	 ;; overwrite-mode $B$G!"%]%$%s%H$,A43QJ8;z$K0O$^$l$F$$$k$H(B
+	 ;; $B$-$K(B delete-backward-char $B$r;H$&$H!"A43QJ8;z$O>C$9$,H>(B
+	 ;; $B3QJ8;zJ,$7$+(B backward $BJ}8~$K%]%$%s%H$,La$i$J$$(B (Emacs
+	 ;; 19.31 $B$K$F3NG'(B)$B!#JQ49Cf$N8uJd$KBP$7$F$O(B
+	 ;; delete-backward-char $B$GI,$:A43QJ8;z(B 1 $BJ8;zJ,(B backward
+	 ;; $BJ}8~$KLa$C$?J}$,NI$$!#(B
+	 (if overwrite-mode
+	     (progn
+	       (backward-char count)
+	       (delete-char count))
+	   ad-do-it)
+	 ;; XXX assume skk-prefix has no multibyte chars.
+	 (if (> (length skk-prefix) count)
+	     (setq skk-prefix (substring skk-prefix
+					 0 (- (length skk-prefix) count)))
+	   (setq skk-prefix ""))
+	 (when (>= skk-henkan-end-point (point))
+	   (if (eq skk-delete-implies-kakutei 'dont-update)
+	       (let ((skk-update-jisyo-function #'ignore))
+		 (skk-kakutei))
+	     (skk-kakutei)))))
+      ((and skk-henkan-mode
+	    (>= skk-henkan-start-point (point))
+	    (not (skk-get-prefix skk-current-rule-tree)))
+       (skk-set-henkan-count 0)
+       (skk-kakutei))
+      ;; $BF~NOCf$N8+=P$78l$KBP$7$F$O(B delete-backward-char $B$G(B
+      ;; $BI,$:A43QJ8;z(B 1$BJ8;zJ,(B backward $BJ}8~$KLa$C$?J}$,NI$$!#(B
+      ((and skk-henkan-mode
+	    overwrite-mode)
+       (backward-char count)
+       (delete-char count))
+      (t
+       (skk-delete-okuri-mark)
+       (if (skk-get-prefix skk-current-rule-tree)
+	   (skk-erase-prefix 'clean)
+	 (skk-set-marker skk-kana-start-point nil)
+	 ad-do-it)))))))
 
 (skk-viper-advice-select
- viper-intercept-ESC-key vip-intercept-ESC-key
+ viper-intercept-ESC-key vip-escape-to-emacs
  (before skk-add activate)
  ("$B"&%b!<%I!""'%b!<%I$@$C$?$i3NDj$9$k!#(B"
   (when (and skk-mode
 	     skk-henkan-mode)
     (skk-kakutei))))
 
+(skk-viper-advice-select
+ viper-intercept-ESC-key vip-escape-to-emacs
+ (after skk-kana-cleanup-ad activate)
+ ("vi-state $B0\9T$N:]$K3NDjF~NO%b!<%I$GF~NO$5$l$?%m!<%^;z%W%l%U%#%C%/%9$r>C$9!#(B"
+  (skk-kana-cleanup t)))
+
 (skk-viper-advice-select
  viper-join-lines vip-join-lines
  (after skk-ad activate)
