@@ -299,9 +299,7 @@
 		      (+ (car P) (/ (1+ fontsize) 2)))
 		  (frame-parameter (selected-frame) 'left)))
 	 (top (+ (cadr edges)
-		 (if (eq situation 'listing)
-		     0
-		   (* 7 (/ (1+ fontsize) 2)))
+		 (* 7 (/ (1+ fontsize) 2))
 		 (or (current-pixel-row window pos)
 		     (+ (cdr P) fontsize))
 		 (frame-parameter (selected-frame) 'top)))
@@ -420,6 +418,8 @@
     (when (and (device-on-window-system-p)
 	       (stringp help))
       (save-excursion
+	(when (find-face skk-tooltip-face)
+	  (setq help (propertize help 'face skk-tooltip-face)))
 	(when (or (not (frame-live-p balloon-help-frame))
 		  (not (eq (selected-device)
 			   (frame-device balloon-help-frame))))
